PRO ASSERT, BOOLEAN_TEST, OUTPUT_STRING

;+
; NAME:
;       ASSERT
;
; PURPOSE:
;       Routine to check a logical condition, and if the condition is
;       NOT TRUE, halt execution with an error message.
;
; CALLING SEQUENCE:
;       assertion, boolean_test, output_string
;
; INPUTS:
;       BOOLEAN_TEST    Logical test for a certain condition
;       OUTPUT_STRING   String to print if the logical test is NOT TRUE
;
; OUTPUTS:
;       If the logical test is NOT TRUE, the contents of OUTPUT_STRING are
;       printed.
;
; PROCEDURE:
;       The IDL MESSAGE procedure is used to produce traceback information
;       on any errors. ON_ERROR is called to return to the calling program.
;
; EXAMPLE:
; a = 5
; assert, ( a ge 6 ), 'A must be GE 6'
;-

rcs_id = "$Id: assert.pro,v 1.1 1998/11/09 19:08:24 gumley Exp $"

;- Return to caller if an error occurs

on_error, 2

;- Check logical test, and if not true, print error message and stop

if not boolean_test then message, output_string
  
END
