PRO dec2dms, dec, deg, min, sec, REVERSE=rev
;+
; NAME:
;	DEC2DMS
; PURPOSE:
;	To convert a value from decimal degrees to deg:min:sec.
; CALLING SEQUENCE:
;	dec2dms, dec, deg, min, sec
; INPUTS:
;	dec     - The decimal number of transform.
; OUTPUTS:
;	deg     - The degrees portion of the transformation.
;	min     - The minutes portion of the transformation.
;	sec     - The seconds portion of the transformation.
; KEYWORDS:
;	REVERSE - If present and non-zero, the reverse transformation
;	          takes place:  a deg:min:sec combination is converted
;	          to decimal deg.  In this case, the INPUTS become
;	          the outputs, and the OUTPUTS become the INPUTS.
; NOTE:
;	This procedure will work with hr:min:sec conversions with no
;	modifications.
; MODIFICATION HISTORY:
;	Written by Michael R. Greason, Hughes STX, 25 November 1992.
;-
;			Check arguments.
;
IF (n_params(0) LT 4) THEN message, 'Syntax:  dec2dms, dec, deg, min, sec'
;
;			Perform the conversion.
;
IF (NOT keyword_set(rev)) THEN BEGIN
;
;				dec -> d:m:s
;
	IF (dec LT 0.) THEN sign = -1. ELSE sign = 1.
	tmp = abs(double(dec))
	deg = double(long(tmp))
	tmp = (tmp - deg) * 60.d
	min = double(long(tmp))
	sec = (tmp - min) * 60.d
	IF (deg GT 0.) THEN deg = deg * sign ELSE $
	IF (min GT 0.) THEN min = min * sign ELSE $
	IF (sec GT 0.) THEN sec = sec * sign
;
ENDIF ELSE BEGIN
;
;				d:m:s -> dec
;
	dec = abs(double(deg)) + abs(double(min) / 60.d) + $
		abs(double(sec) / 3600.d)
	IF ((deg LT 0.) OR (min LT 0.) OR (sec LT 0.)) THEN dec = -dec
;
ENDELSE
;
RETURN
END
