FUNCTION DIALOG_INPUTFILE, DIALOG_PARENT = DIALOG_PARENT, $
  GET_PATH = GET_PATH, _EXTRA = EXTRA

;+
; Purpose:
;    DIALOG_INPUTFILE allows the user to interactively pick an input file
;    using the platform's own native graphical file-selection dialog.
;    Features:
;    (1) If the user enters the name of a non-existent file, an error
;        message is printed, and the user is asked to select a different
;        filename.
;    (2) The selected input file is checked to make sure it can be read.
;        If not, the user is warned and asked to select another file.
;
; Input:
;    None
;
; Output:
;    DIALOG_INPUTFILE     String containing the full path name of the
;                         selected input file.
;
; Optional Keywords:
;    DIALOG_PARENT        Set this keyword to the widget ID of a widget
;                         over which any message dialogs should be positioned.
;
;    Also accepts any keywords recognized by DIALOG_PICKFILE.
;
; Example:
;
; Author:
;    Liam.Gumley@ssec.wisc.edu
;-

rcs_id = "$Id: dialog_inputfile.pro,v 1.2 1999/06/17 21:43:06 gumley Exp $"

;- Check keywords

if n_elements( dialog_parent ) ne 1 then dialog_parent = 0

;- Call dialog_pickfile to select a file name (return if no file selected)
;  (note that get_path is included because extra keyword values are not returned)

select_file:
filename = dialog_pickfile( get_path = get_path, _extra = extra )
if filename eq '' then return, filename

;- Test whether file exists

if ( findfile( filename ) )[0] eq '' then begin

  ;- File does not exist, so print an error message and go select another file

  text = filename + ' does not exist. Please use a different filename.'
  result = dialog_message( text, /error, dialog_parent = dialog_parent )
  goto, select_file

endif else begin

  ;- File exists, so check that it can be read

  if not filecheck( filename ) then begin
    text = filename + ' could not be opened. Please use a different filename.'
    result = dialog_message( text, /error, dialog_parent = dialog_parent )
    goto, select_file
  endif

endelse

;- Return result to caller

return, filename

END
