FUNCTION FILECHECK, FILE, WRITE = WRITE

;+
; Purpose:
;    To check if a file is can be opened for reading, updating, or writing.
;
; Input:
;    FILE         String containing the full path and name of the file
;
; Output:
;    FILECHECK    Integer flag indicating success (1) or failure (0)
;
; Optional Keywords:
;    WRITE        If set, check if the file is available for updating or writing
;                 (default is check if the file is available for reading)
;
; Example:
; file = 'zztest.dat'
; print, filecheck( file )
; print, filecheck( file, /write )
;
; Author:
;    Liam.Gumley@ssec.wisc.edu
;-

rcs_id = "$Id: filecheck.pro,v 1.2 1999/06/17 21:43:05 gumley Exp $"

;- Check arguments

if n_elements( file ) eq 0 then message, 'Argument FILE is missing'

;- Check keywords

if not keyword_set( write ) then write = 0

;- Set default return value

result = 0

;- Check if file exists

if ( findfile( file ) )[0] eq '' then begin

  ;- File does not exist, so if WRITE set, check if it can be opened for write

  if write then begin
    openw, lun, file, /get_lun, /delete, error = error
    if error eq 0 then begin
      result = 1
      free_lun, lun
    endif
  endif
  
endif else begin

  ;- File exists, so check if it can be opened for update or read

  if write then begin
    openu, lun, file, /get_lun, error = error
  endif else begin
    openr, lun, file, /get_lun, error = error
  endelse
  if error eq 0 then begin
    result = 1
    free_lun, lun
  endif

endelse

;- Return result to caller

return, result

END
