;###############################################################################

pro formula_compute, path, band1, band2, band3, band4, $
  operator1, operator2, operator3, result, scene = scene, units = units
  
;- check keywords

if n_elements( scene ) ne 2 then scene = [ 0, 715 ]
if not keyword_set( units ) then units = 0

;- set default return value

result = -1.0

;- get first band

band1 = ( band1 > 1 ) < 50
mas_read, path, band1, band1_data, date, time1, time2, scene = scene, units = units

;- if left operator was selected, read second band and compute left formula

if operator1 ne 0 then begin

  ;- get second band

  band2 = ( band2 > 1 ) < 50
  mas_read, path, band2, band2_data, date, time1, time2, scene = scene, units = units

  ;- compute formula

  case operator1 of
    1 : data = temporary( band1_data ) + temporary( band2_data )
    2 : data = temporary( band1_data ) - temporary( band2_data )
    3 : data = temporary( band1_data ) * temporary( band2_data )
    4 : begin
          sz = size( band1_data )
          data = replicate( -999.0, sz( 1 ), sz( 2 ) )
          loc = where( abs( band2_data + 1.0 ) gt 1.0, count )
          if count ge 1 then data( loc ) = $
            temporary( band1_data( loc ) ) / temporary( band2_data( loc ) )
    end
  end

endif

;- if center operator was not selected, then return result

if operator2 eq 0 then begin
  if operator1 eq 0 then begin
    result = temporary( band1_data )
  endif else begin
    result = temporary( data )
  endelse
  return
endif

;- save left data

if operator1 eq 0 then begin
  data_left = temporary( band1_data )
endif else begin
  data_left = temporary( data )
endelse

;- get third band

band3 = ( band3 > 1 ) < 50
mas_read, path, band3, band3_data, date, time1, time2, scene = scene, units = units

;- if right operator was selected, read fourth band and compute right formula

if operator3 ne 0 then begin

  ;- get fourth band

  band4 = ( band4 > 1 ) < 50
  mas_read, path, band4, band4_data, date, time1, time2, scene = scene, units = units
  
  ;- compute formula
  
  case operator3 of
    1 : data = temporary( band3_data ) + temporary( band4_data )
    2 : data = temporary( band3_data ) - temporary( band4_data )
    3 : data = temporary( band3_data ) * temporary( band4_data )
    4 : begin
          sz = size( band3_data )
          data = replicate( -999.0, sz( 1 ), sz( 2 ) )
          loc = where( abs( band4_data + 1.0 ) gt 1.0, count )
          if count ge 1 then data( loc ) = $
            temporary( band3_data( loc ) ) / temporary( band4_data( loc ) )
    end
  end

endif

;- save right data

if operator3 eq 0 then begin
  data_right = temporary( band3_data )
endif else begin
  data_right = temporary( data )
endelse

;- compute complete formula

case operator2 of
  1 : data = temporary( data_left ) + temporary( data_right )
  2 : data = temporary( data_left ) - temporary( data_right )
  3 : data = temporary( data_left ) * temporary( data_right )
  4 : begin
        sz = size( data_left )
        data = replicate( -999.0, sz( 1 ), sz( 2 ) )
        loc = where( abs( data_right + 1.0 ) gt 1.0, count )
        if count ge 1 then $
          data( loc ) = temporary( data_left( loc ) ) / temporary( data_right( loc ) )
  end
end

;- save result

result = temporary( data )

end
  
;###############################################################################

pro formula_event, event

;- get info structure

widget_control, event.top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- get user value from widget causing event

widget_control, event.id, get_uvalue = uvalue

;- check possible events

case uvalue of

  'Operator1' : begin
    info.operator1 = event.index
    if event.index ne 0 then begin
      widget_control, info.band2_id, sensitive = 1
    endif else begin
      widget_control, info.band2_id, sensitive = 0
    endelse
  end
    
  'Operator2' : begin
    info.operator2 = event.index
    if event.index ne 0 then begin
      widget_control, info.base3, sensitive = 1
    endif else begin
      widget_control, info.base3, sensitive = 0
    endelse
  end

  'Operator3' : begin
    info.operator3 = event.index
    if event.index ne 0 then begin
      widget_control, info.band4_id, sensitive = 1
    endif else begin
      widget_control, info.band4_id, sensitive = 0
    endelse
  end

  'Cancel' : widget_control, event.top, /destroy

  'OK' : begin
    widget_control, info.band1_id, get_value = band1
    info.band1 = band1
    widget_control, info.band2_id, get_value = band2
    info.band2 = band2
    widget_control, info.band3_id, get_value = band3
    info.band3 = band3
    widget_control, info.band4_id, get_value = band4
    info.band4 = band4
    info.ok = 1
    widget_control, event.top, hourglass = 1
    formula_compute, info.path, info.band1, info.band2, info.band3, info.band4, $
      info.operator1, info.operator2, info.operator3, result, $
      scene = info.scene, units = info.units
    info.result = temporary( result )
    widget_control, event.top, hourglass = 0
    widget_control, event.top, /destroy
  end
      
  else :
  
end

;- update info structure

pointer_value, pointer, info, /set, /no_copy

end

;###############################################################################

PRO FORMULA, PATH, RESULT, GROUP = GROUP, $
  XOFFSET = XOFFSET, YOFFSET = YOFFSET, $
  BANDS = BANDS, OPERATORS = OPERATORS, TITLE = TITLE, SCENE = SCENE, UNITS = UNITS

rcs_id = "$Id: formula.pro,v 1.7 1999/09/14 19:47:36 gumley Exp $"

result = -1.0
if n_elements( group ) eq 0 then group = 0
if n_elements( xoffset ) eq 0 then xoffset = !d.x_vsize / 4
if n_elements( yoffset ) eq 0 then yoffset = !d.y_vsize / 2
if n_elements( bands ) eq 0 then bands = [ 1, 2, 3, 4 ]
if n_elements( operators ) eq 0 then operators = [ 0, 0, 0 ]
if n_elements( title ) eq 0 then title = ""
if n_elements( scene ) ne 2 then scene = [ 0, 715 ]
if not keyword_set( units ) then units = 0

;- Create top level base

tlb = widget_base( title = 'Band Math', col = 1, $
  group = group, xoffset = xoffset, yoffset = yoffset, /modal )

;- Create formula widgets

base_top = widget_base( tlb, row = 1 )

base1 = widget_base( base_top, row = 1, /frame )
base2 = widget_base( base_top, row = 1, /frame )
base3 = widget_base( base_top, row = 1, /frame )

value = [ '   ', ' + ', ' - ', ' * ', ' / ' ]

band1_id = cw_field( base1, value = bands( 0 ), uvalue = 'Band1', $
  xsize = 4, title = 'Band', /integer, /all_events )

operator1_id = widget_droplist( base1, uvalue = 'Operator1', value = value )

band2_id = cw_field( base1, value = bands( 1 ), uvalue = 'Band2', $
  xsize = 4, title = 'Band', /integer, /all_events )

operator2_id = widget_droplist( base2, uvalue = 'Operator2', value = value )

band3_id = cw_field( base3, value = bands( 2 ), uvalue = 'Band3', $
  xsize = 4, title = 'Band', /integer, /all_events )

operator3_id = widget_droplist( base3, uvalue = 'Operator3', value = value )

band4_id = cw_field( base3, value = bands( 3 ), uvalue = 'Band4', $
  xsize = 4, title = 'Band', /integer, /all_events )

;- Create action button widgets

base_bottom = widget_base( tlb, row = 1, /align_center )
buttsize = 60
b = widget_button( base_bottom, value = 'OK', uvalue = 'OK', xsize = buttsize )
b = widget_button( base_bottom, value = 'Cancel', uvalue = 'Cancel', xsize = buttsize )

;- Realize widgets

widget_control, tlb, /realize

;- desensitize unselected widgets

widget_control, operator1_id, set_droplist_select = operators( 0 )
widget_control, operator2_id, set_droplist_select = operators( 1 )
widget_control, operator3_id, set_droplist_select = operators( 2 )
if operators( 0 ) eq 0 then widget_control, band2_id, sensitive = 0
if operators( 1 ) eq 0 then widget_control, base3, sensitive = 0
if operators( 2 ) eq 0 then widget_control, band4_id, sensitive = 0

;- create and store info structure

info = { path      : path, $
         scene     : scene, $
         units     : units, $
         base3     : base3, $
         operator1 : operators( 0 ), $
         operator2 : operators( 1 ), $
         operator3 : operators( 2 ), $
         band1_id  : band1_id, $
         band2_id  : band2_id, $
         band3_id  : band3_id, $
         band4_id  : band4_id, $
         band1     : bands( 0 ), $
         band2     : bands( 1 ), $
         band3     : bands( 2 ), $
         band4     : bands( 3 ), $
         result    : fltarr( 716, 716, /nozero ), $
         ok        : 0 }
pointer_create, pointer
pointer_value, pointer, info, /set, /no_copy
widget_control, tlb, set_uvalue = pointer
         
;- start event manager

xmanager, "formula", tlb

;- retrieve info structure

pointer_value, pointer, info, /no_copy
pointer_free, pointer

;- save result

if info.ok eq 1 then begin

  result = info.result
  bands = [ info.band1, info.band2, info.band3, info.band4 ]
  operators = [ info.operator1, info.operator2, info.operator3 ]

  fmt = '( "B", i2.2 )'
  title = string( bands( 0 ), format = fmt )
  if operators( 0 ) ne 0 then begin
    title = title + value( operators( 0 ) )
    title = title + string( bands( 1 ), format = fmt )
  endif
  title = "(" + title + ")"

  if operators( 1 ) ne 0 then begin
    title = title + value( operators( 1 ) ) + "("
    title = title + string( bands( 2 ), format = fmt )
    if operators( 2 ) ne 0 then begin
      title = title + value( operators( 2 ) )
      title = title + string( bands( 3 ), format = fmt )
    endif
    title = title + ")"
  endif
  
  title = strcompress( title, /remove_all )

endif

END

;###############################################################################
