FUNCTION GPL, TITLE, YEAR, AUTHOR

;+
; Purpose:
;    To construct a string array that be displayed using XDISPLAYFILE
;    that contains the GNU Public license.
;
; Input:
;    TITLE    A string containing the title of the work
;    YEAR     A string containing the year of publication
;    AUTHOR   A string containing the full name of the author
;
; Output:
;    GPL      A string array containing the GNU Public license for this work
;
; Example:
;    text = gpl( 'A Viewer for MAS data', 1998, 'Liam E. Gumley' )
;    xdisplayfile, text=text, done='Done', title='About this program'
;
; Author:
;    Liam.Gumley@ssec.wisc.edu
;-

rcs_id = "$Id: gpl.pro,v 1.2 1998/12/21 18:33:02 gumley Exp $"

;- Check arguments

if n_elements( title )  eq 0 then message, 'Argument TITLE is missing'
if n_elements( year )   eq 0 then message, 'Argument YEAR is missing'
if n_elements( author ) eq 0 then message, 'Argument AUTHOR is missing'

;- Create string array

text = [ strcompress( title ) ]

text = [ text, 'Copyright (C) ' + strcompress( year, /remove ) + ' ' + strcompress( author ) ]

text = [ text, $
  ' ', $
  'This program is free software; you can redistribute it and/or', $
  'modify it under the terms of the GNU General Public License', $
  'as published by the Free Software Foundation; either version 2', $
  'of the License, or (at your option) any later version.' ]

text = [ text, $
  ' ', $
  'This program is distributed in the hope that it will be useful,', $
  'but WITHOUT ANY WARRANTY; without even the implied warranty of', $
  'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the', $
  'GNU General Public License for more details.' ]

text = [ text, $
  ' ', $
  'You should have received a copy of the GNU General Public License', $
  'along with this program; if not, write to the Free Software', $
  'Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.' ]

return, text

END
