PRO HIS_READ, FILE, MAS_DATE, HIS_TIME

rcs_id = "$Id: his_read.pro,v 1.2 1998/07/24 18:10:46 gumley Exp $"

;+
; Purpose:
;    Read a HIS file and make sure it corresponds to a given MAS date.
;    Invalid HIS files will be identified and rejected.
;    Byte swapping is done automatically if needed.
;
; Usage:
;    HIS_READ, FILE, MAS_DATE, HIS_TIME
;
; Input:
;    FILE        Name of HIS file
;    MAS_DATE    MAS date (YYYYMMDD)
;
; Output:
;    HIS_TIME    Array of all HIS times (HHMMSS)
;-

;- set default return value

his_time = -1L

;- read first HIS record and swap bytes if necessary

his_record = his_declare_file_structure()
openr, his_lun, file, /get_lun
readu, his_lun, his_record
free_lun, his_lun
swap = 0
if long( his_record.channel ) lt 1 or long( his_record.channel ) gt 3 then begin
  his_record = swap_endian( his_record )
  swap = 1
endif

;- extract first HIS date (YYYYMMDD) and time (HHMMSS)

year = long( his_record.year ) + 1900L
if year lt 1980 then year = year + 100L
month = long( his_record.month )
day = long( his_record.day )
his_date1 = year * 10000L + month * 100L + day
his_time1 = long( his_record.current_time )
his_channel = long( his_record.channel )

;- check first HIS date, time, and channel for validity

if his_date1 lt 19800101L or his_date1 gt 20101231L or $
  his_time1 lt 0L or his_time1 gt 235959L or $
  his_channel lt 1 or his_channel gt 3 then begin
  string = string( file, format = '( "Sorry: ", a, " is not a valid HIS file" )' )
  result = widget_message( string )
  return
endif

;- read entire HIS data file and swap bytes

openr, his_lun, file, /get_lun
file_status = fstat( his_lun )
n_rec = file_status.size / ( 4L * 2150L )
his_data = make_array( n_rec, value = his_record )
readu, his_lun, his_data
free_lun, his_lun
if swap then his_data = swap_endian( temporary( his_data ) )

;- extract HIS dates

year = long( his_data(*).year ) + 1900L
loc = where( year lt 1980, count )
if count ge 1 then year( loc ) = year( loc ) + 100L
month = long( his_data(*).month )
day = long( his_data(*).day )
his_date = year * 10000L + month * 100L + day

;- check that MAS date and time are found within the HIS file

loc = where( his_date eq mas_date, count )
if count lt 1 then begin
  string = string( his_date(0), mas_date, format = $
    '( "Sorry: HIS date (", i8, ") does not match MAS date (", i8, ")" )' )
  result = widget_message( string )
  return
endif

;- extract all HIS times (HHMMSS)

his_time = long( temporary( his_data(*).current_time ) )
his_data = 0B

END
