FUNCTION LINEARSTRETCH, IMAGE, CUTOFF = CUTOFF

rcs_id = "$Id: linearstretch.pro,v 1.6 1999/01/06 20:04:27 gumley Exp $"

;- Set return value

returnvalue = -1

;- Check arguments

assert, ( n_elements( image ) ne 0 ), 'Argument IMAGE is missing'

;- Check keywords

if n_elements( cutoff ) eq 0 then cutoff = 0.02
assert, ( cutoff ge 0.001 and cutoff le 0.4 ), 'Keyword CUTOFF not in range [0.001,0.4]'

;- Get image minimum and maximum

minvalue = min( image, max = maxvalue )

;- Check image range

if not ( minvalue lt maxvalue ) then return, returnvalue

;- Define number of points used in constructing histogram

points = 100

;- Compute histogram

binsize = ( float( maxvalue ) - float( minvalue ) ) / float( points - 1 )
histdata = histogram( float( image ), min = minvalue, max = maxvalue, $
  binsize = binsize )
if n_elements( histdata ) lt 2 then return, returnvalue

;- Compute corresponding data ordinates

ordinate = minvalue + $
  ( maxvalue - minvalue ) * findgen( points ) / float( points - 1 )

;- Create normalized cumulative sum from histogram

sum = histdata
for i = 1, n_elements( histdata ) - 1 do sum[ i ] = sum[ i - 1 ] + histdata[ i ]
normalsum = float( sum ) / float( n_elements( image ) )

;- Find the cutoff at the lower and upper ends of the cumulative sum array

location = where( normalsum ge cutoff and normalsum le ( 1.0 - cutoff ), count )
if count ge 2 then begin
  stretchmin = ordinate[ location[ 0 ] ]
  stretchmax = ordinate[ location[ count - 1 ] ]
  returnvalue = [ stretchmin, stretchmax ]  
endif

return, returnvalue

END
