PRO MAS_DISPLAY, INFO, TITLE = TITLE, REGION = REGION, MASK_ONLY = MASK_ONLY, $
  MAP = MAP

rcsid   = "$Id: mas_display.pro,v 1.15 1999/06/15 15:13:00 gumley Exp $"

@sharp.inc
xoffset = 0
yoffset = 0

;- Check keywords

if not keyword_set( title ) or info.title eq " " then $
  title = strcompress( info.band, /remove_all )
if not keyword_set( region ) then region = [ 0.0, 0.0, 1.0, 1.0 ]
if not keyword_set( mask_only ) then mask_only = 0

;- Set display order

!order = 0

;- If we are only updating the mask, skip the image creation

if mask_only then goto, mask

;- Create the byte scaled image

case info.rgb_flag of

  ;- create single band byte scaled image
  
  0 : begin
    bytimg = bytscl( info.image, min = info.minv, max = info.maxv, $
      top = !d.table_size - 1 - 16 ) + 16B
  end
  
  ;- create three band RGB byte scaled image

  1 : begin

    rgbimg = bytarr( 3, 716, 716 )
    if info.rgb_equalize then begin

      minv = min( info.rgb_image( 0, *, * ), max = maxv )
      rgbimg( 0, *, * ) = hist_equal( info.rgb_image( 0, *, * ), $
	  minv = minv, maxv = maxv, top = !d.table_size - 1 )
      minv = min( info.rgb_image( 1, *, * ), max = maxv )
      rgbimg( 1, *, * ) = hist_equal( info.rgb_image( 1, *, * ), $
	  minv = minv, maxv = maxv, top = !d.table_size - 1 )
      minv = min( info.rgb_image( 2, *, * ), max = maxv )
      rgbimg( 2, *, * ) = hist_equal( info.rgb_image( 2, *, * ), $
	  minv = minv, maxv = maxv, top = !d.table_size - 1 )

    endif else begin

      ;- compute range for 2% linear stretch
      
      stretchrange = linearstretch( info.rgb_image( 0, *, * ) )
      if n_elements( stretchrange ) eq 2 then begin
        minv = stretchrange( 0 )
        maxv = stretchrange( 1 )
      endif else begin
        minv = min( info.rgb_image( 0, *, * ), max = maxv )
      endelse
      rgbimg( 0, *, * ) = bytscl( info.rgb_image( 0, *, * ), $
        min = minv, max = maxv )

      stretchrange = linearstretch( info.rgb_image( 1, *, * ) )
      if n_elements( stretchrange ) eq 2 then begin
        minv = stretchrange( 0 )
        maxv = stretchrange( 1 )
      endif else begin
        minv = min( info.rgb_image( 1, *, * ), max = maxv )
      endelse
      rgbimg( 1, *, * ) = bytscl( info.rgb_image( 1, *, * ), $
        min = minv, max = maxv )

      stretchrange = linearstretch( info.rgb_image( 2, *, * ) )
      if n_elements( stretchrange ) eq 2 then begin
        minv = stretchrange( 0 )
        maxv = stretchrange( 1 )
      endif else begin
        minv = min( info.rgb_image( 2, *, * ), max = maxv )
      endelse
      rgbimg( 2, *, * ) = bytscl( info.rgb_image( 2, *, * ), $
        min = minv, max = maxv )

    endelse

    ;- Flip grey scale if requested
    
    if info.rgb_red_flip then begin
      high = max( rgbimg( 0, *, * ) )
      rgbimg( 0, *, * ) = high - rgbimg( 0, *, * )
    endif
    
    if info.rgb_grn_flip then begin
      high = max( rgbimg( 1, *, * ) )
      rgbimg( 1, *, * ) = high - rgbimg( 1, *, * )
    endif
    
    if info.rgb_blu_flip then begin
      high = max( rgbimg( 2, *, * ) )
      rgbimg( 2, *, * ) = high - rgbimg( 2, *, * )
    endif

    ;- Create pseudocolor image
    
    bytimg = color_quan( temporary( rgbimg ), 1, r, g, b, $
      colors = !d.table_size - 16 ) + 16B
    tvlct, r, g, b, 16

  end
  
endcase

;- if map flag is on, then convert image to map projection

if info.map_flag then begin

  ;- set image position
  
  pos = [ 0.0, 0.0, 716.0/draw_xsize, 716.0/draw_ysize ]
  
  ;- create byte scaled image, or render directly to Postscript
  
  if !d.name ne "PS" then begin

    ;- switch to Z buffer and create map

    entry_device = !d.name
    ncolors = !d.table_size
    x_ch_size = !d.x_ch_size
    y_ch_size = !d.y_ch_size
    set_plot, 'Z'
    device, z_buffer=0, set_colors=ncolors, set_resolution=[draw_xsize, draw_ysize], $
      set_character_size=[x_ch_size, y_ch_size]
    erase, 7
    imagemap, bytimg, info.map_lat, info.map_lon, /isotropic, pos = pos, $
	missing = 7B, /noborder, ncolors = !d.table_size - 16, bottom = 16B, $
	/noerase
    bytimg = tvrd(0, 0, 716, 716)
    set_plot, entry_device

    ;- display the image in the draw pixmap

    wset, info.draw_pixmap
    tv, bytimg

  endif else begin

    missing = byte( !d.table_size - 1 )
    if info.rgb_flag then missing = 7B
    pos = [ convert_region( pos( 0 ), pos( 1 ), region ), $
      convert_region( pos( 2 ), pos( 3 ), region ) ]
    imagemap, bytimg, info.map_lat, info.map_lon, /isotropic, pos = pos, $
	missing = missing, /noborder, ncolors = !d.table_size - 16, bottom = 16B

  endelse  
    
  ;- get lat/lon limits
  
  latmin = min( info.map_lat, max = latmax )
  lonmin = min( info.map_lon, max = lonmax )
    
  ;- create latitude grid

  n = ( ceil( latmax ) - floor( latmin ) ) * 10.0
  lats = findgen( n ) / float( n )
  lats = lats * ( ceil( latmax ) - floor( latmin ) ) + floor( latmin )
  loc = where( lats gt latmin and lats lt latmax, count )
  if count ge 1 then begin
    lats_label = lats( loc )
  endif else begin
    lats_label = [ latmin, latmax ]
  endelse
  
  ;- create longitude grid

  n = ( ceil( lonmax ) - floor( lonmin ) ) * 10.0
  lons = findgen( n ) / float( n )
  lons = lons * ( ceil( lonmax ) - floor( lonmin ) ) + floor( lonmin )
  loc = where( lons gt lonmin and lons lt lonmax, count )
  if count ge 1 then begin
    lons_label = lons( loc )
  endif else begin
    lons_label = [ lonmin, lonmax ]
  endelse

  ;- save map setup
  
  xinfo = !x
  yinfo = !y
  pinfo = !p
  minfo = !map
  
endif

;- display image in draw pixmap

if !d.name ne 'PS' then begin
  wset, info.draw_pixmap
  erase, 7
  tv, bytimg, xoffset, yoffset
endif

;- if we are only updating the cloud mask, then grab byte image from draw pixmap

mask:
if mask_only and !d.name ne "PS" then begin
  wset, info.draw_pixmap
  bytimg = temporary( safe_tvrd() )
  bytimg = temporary( bytimg( 0:715, 0:715 ) )
endif

;- if necessary, display image with cloud mask overlay in mask pixmap

if info.mask_flag and info.mask_path ne " " then begin

  ;- cloud mask results
  
  newimg = bytimg
  loc = where( info.mask_select eq 1, count )
  if count ge 1 then begin
    test = info.mask_test( loc )
    color = info.mask_color( loc )
    newimg = mask_image( temporary( newimg ), info.mask_data, test, color, $
      names = result_names  )
    newimg( *, 0 )   = bytimg( *, 0 )
    newimg( *, 715 ) = bytimg( *, 715 )
    newimg( 0, * )   = bytimg( 0, * )
    newimg( 715, * ) = bytimg( 715, * )
    if info.mask_toggle then bytimg = newimg
  endif
  if !d.name ne 'PS' then begin
    wset, info.mask_pixmap
    erase, 7
    tv, newimg, xoffset, yoffset
  endif
  
  ;- cloud mask test overlay
  
  test = info.mask_list
  color = 1
  newimg = mask_image( temporary( newimg ), info.mask_data, test, color, $
    names = test_names )
  newimg( *, 0 )   = bytimg( *, 0 )
  newimg( *, 715 ) = bytimg( *, 715 )
  newimg( 0, * )   = bytimg( 0, * )
  newimg( 715, * ) = bytimg( 715, * )
  if info.mask_toggle1 then bytimg = newimg
  if !d.name ne 'PS' then begin
    wset, info.mask_pixmap1
    erase, 7
    tv, temporary( newimg ), xoffset, yoffset
  endif

endif

;- display image in draw window, or display Postscript image

if !d.name ne "PS" then begin
  wset, info.draw_window
  erase, 7
  tv, temporary( bytimg ), xoffset, yoffset
endif else begin
  if not info.map_flag then begin
    pos = [ 0.0, 0.0, 716.0 / draw_xsize, 716.0 / draw_ysize ]
    pos = [ convert_region( pos( 0 ), pos( 1 ), region ), $
        convert_region( pos( 2 ), pos( 3 ), region ) ]
    imdisp, temporary( bytimg ), pos = pos, /usepos
  endif
endelse

;- make sure bottom graphics index is black

if !d.name eq "PS" then tvlct, 0B, 0B, 0B, 0

;- draw colorbar

if info.rgb_flag ne 1 then begin

  fmt = '( f7.2 )'
  if ( info.maxv - info.minv ) lt 0.1 then fmt = '(f8.3)'
  if info.band eq 99 then begin
    string = title
  endif else begin
    string = 'Reflectance'
    if info.center( info.band - 1 ) ge 2.5 then begin
	string = 'Brightness Temp.'
    endif
  endelse
  scale = 716.0 / draw_ysize
  pos = [ 0.86, scale * 0.175, 0.92, scale * 0.9 ]
  if info.mask_flag and info.mask_path ne " " then pos = [ 0.86, scale * 0.175, 0.92, scale * 0.7 ]
  pos = [ convert_region( pos( 0 ), pos( 1 ), region ), $
          convert_region( pos( 2 ), pos( 3 ), region ) ]
  colorbar, /vertical, /right, min = info.minv, max = info.maxv, $
    charsize = 1.5, pos = pos, $
    bottom = 16, ncolors = !d.table_size - 16, color = 0, format = fmt

endif

;- draw start and end time labels next to image

if not info.map_flag then begin

  time = string( info.time1, format = '(i6.6)' )
  time = strmid( time, 0, 2 ) + ":" + $
         strmid( time, 2, 2 ) + ":" + $
         strmid( time, 4, 2 ) + " UTC"

  x = 716.0 / draw_xsize + 0.005
  y = 0.005
  p = convert_region( x, y, region )
  xyouts, p( 0 ), p( 1 ), time, /normal, align = 0.0, charsize = 1.5, color = 0

  time = string( info.time2, format = '(i6.6)' )
  time = strmid( time, 0, 2 ) + ":" + $
         strmid( time, 2, 2 ) + ":" + $
         strmid( time, 4, 2 ) + " UTC"

  x = 716.0 / draw_xsize + 0.005
  y = 0.955
  p = convert_region( x, y, region )   
  xyouts, p( 0 ), p( 1 ), time, /normal, align = 0.0, charsize = 1.5, color = 0

endif

;- Print image orientation information

if !d.name ne 'PS' then select_font, size = 12, /bold

relazm1  = abs( 180.0 - abs( info.senazm(   0, 357 ) - info.solazm(   0, 357 ) ) )
relazm2 = abs( 180.0 - abs( info.senazm( 715, 357 ) - info.solazm( 715, 357 ) ) )
relazm = relazm1 < relazm2
values = [ info.map_lat( 357, 357 ), info.map_lon( 357, 357 ), info.heading( 357 ), $
  info.solzen( 357, 357 ), info.solazm( 357, 357 ), relazm ]

labels = [ 'Latitude: ', 'Longitude: ', 'Heading: ', $
  'Solar Zen: ', 'Solar Azim: ', 'Rel. Azim: ' ]

x = 716.0 / draw_xsize + 0.005
y = 0.15

for i = 0, n_elements( labels ) - 1 do begin
  labelstring = labels( i ) + string( values( i ), format = '(f6.1)' )
  p = convert_region( x, y, region )
  xyouts, p( 0 ), p( 1 ), labelstring, /normal, align = 0.0, charsize = 1.1, color = 0
  y = y - 0.02
endfor

if !d.name ne 'PS' then select_font, size = 14, /bold

;- draw title

string = string( info.band, format = '( "Band ", i2.2 )' )
if info.band le 50 then begin
  string = string + " (" + $
    strcompress( string( info.center( info.band - 1 ), format = '( f5.2 )' ), /remove_all ) + $
    " micron)"
  if info.units_flag then begin
    string = string + ' Radiance (W/m!U2!N/sr/micron)'
  endif else begin
    if info.center( info.band - 1 ) le 2.5 then begin
	string = string + ' Reflectance'
    endif else begin
	string = string + ' Brightness Temp. (K)'
    endelse
  endelse  
endif else begin
  string = title
endelse

if info.rgb_flag then string = string( info.rgb_bands, $
  format = '( "R:", i2.2, ",G:",i2.2,",B:",i2.2 )' )

date = strmid( info.date, 0, 4 ) + "/" + $
       strmid( info.date, 4, 2 ) + "/" + $
       strmid( info.date, 6, 2 ) 

time = string( info.time( 357 ), format = '(i6.6)' )
time = " " + $
       strmid( time, 0, 2 ) + ":" + $
       strmid( time, 2, 2 ) + ":" + $
       strmid( time, 4, 2 ) + " UTC"

track = string( info.track_number, format = '( " Track ", i2.2, ", " )' )

string = "MAS (" + info.expname + ") " + $
  date + time + track + string

x = 0.5
y = 716.0 / draw_ysize + 0.0075
p = convert_region( x, y, region )   
xyouts, p( 0 ), p( 1 ), string, /normal, align = 0.5, charsize = 1.5, color = 0

;- save titles and colorbar in pixmaps

if !d.name ne "PS" then begin
  wset, info.draw_pixmap
  device, copy = [ 716, 0, draw_xsize - 716, draw_ysize, 716, 0, info.draw_window ]
  device, copy = [ 0, 716, 716, draw_ysize - 716, 0, 716, info.draw_window ]
  wset, info.mask_pixmap
  device, copy = [ 716, 0, draw_xsize - 716, draw_ysize, 716, 0, info.draw_window ]
  device, copy = [ 0, 716, 716, draw_ysize - 716, 0, 716, info.draw_window ]
  wset, info.mask_pixmap1
  device, copy = [ 716, 0, draw_xsize - 716, draw_ysize, 716, 0, info.draw_window ]
  device, copy = [ 0, 716, 716, draw_ysize - 716, 0, 716, info.draw_window ]
  wset, info.draw_window
endif

;- cloud mask legend

if info.mask_flag and info.mask_path ne " " and info.mask_toggle then begin

  if !d.name ne 'PS' then select_font, size = 12, /bold
    
  ;- cloud mask results

  if !d.name ne 'PS' then wset, info.mask_pixmap

  x = 716.0 / draw_xsize + 0.005
  y = 0.9125
  loc = where( info.mask_select eq 1, count )

  if count ge 1 then for i = 0, count - 1 do begin

    p = convert_region( x, y, region )   
    xcoord = [ x, x + 0.02, x + 0.02, x ]
    ycoord = [ y, y, y + 0.02, y + 0.02 ]
    pcoord = convert_region( xcoord, ycoord, region )
    polyfill, pcoord( 0 : 3 ), pcoord( 4 : 7 ), /normal, $
      color = info.mask_color( loc( i ) )
    plots, [ pcoord( 0 : 3 ), p( 0 ) ], [ pcoord( 4 : 7 ), p( 1 ) ], $
      /normal, color = 0
    p = convert_region( x + 0.025, y + 0.005, region )
    xyouts, p( 0 ), p( 1 ), result_names( i ), /normal, $
      color = 0, charsize = 1.1
    y = y - 0.04

  endfor

  if !d.name ne 'PS' then begin
    wset, info.draw_window
    device, copy = [ 716, 0, draw_xsize - 716, draw_ysize, 716, 0, info.mask_pixmap ]
    wset, info.mask_pixmap1
    device, copy = [ 716, 0, draw_xsize - 716, draw_ysize, 716, 0, info.mask_pixmap ]
  endif

  ;- cloud mask test overlay

  if !d.name eq 'PS' then begin
    if not info.mask_toggle1 then goto, continue
  endif else begin
    wset, info.mask_pixmap1
  endelse
  
  color = 1
  p = convert_region( x, y, region )   
  xcoord = [ x, x + 0.02, x + 0.02, x ]
  ycoord = [ y, y, y + 0.02, y + 0.02 ]
  pcoord = convert_region( xcoord, ycoord, region )
  polyfill, pcoord( 0 : 3 ), pcoord( 4 : 7 ), /normal, color = color
  plots, [ pcoord( 0 : 3 ), p( 0 ) ], [ pcoord( 4 : 7 ), p( 1 ) ], $
    /normal, color = 0
  p = convert_region( x + 0.025, y + 0.005, region )
  text = test_names( 0 )

  if info.mask_list ge 18 then text = text + '!CFound Cloud'
  xyouts, p( 0 ), p( 1 ), text, /normal, color = 0, charsize = 1.1
  
  if info.mask_list ge 18 then begin
    y = y - 0.04
    color = 3
    p = convert_region( x, y, region )   
    xcoord = [ x, x + 0.02, x + 0.02, x ]
    ycoord = [ y, y, y + 0.02, y + 0.02 ]
    pcoord = convert_region( xcoord, ycoord, region )
    polyfill, pcoord( 0 : 3 ), pcoord( 4 : 7 ), /normal, color = color
    plots, [ pcoord( 0 : 3 ), p( 0 ) ], [ pcoord( 4 : 7 ), p( 1 ) ], $
      /normal, color = 0
    p = convert_region( x + 0.025, y + 0.005, region )
    xyouts, p( 0 ), p( 1 ), test_names( 0 ) + '!CTest Not Run', /normal, $
      color = 0, charsize = 1.1
  endif
    
  continue:
  
  if !d.name ne 'PS' then select_font, size = 14, /bold
  
endif

;- update map information

if info.map_flag then begin

  if !d.name ne 'PS' then wset, info.draw_pixmap
  
  ;- restore map setup
  
  !x = xinfo
  !y = yinfo
  !p = pinfo
  !map = minfo

  ;- plot map reference marks at intervals of 0.1 degrees
  
  for j = 0, n_elements( lats ) - 1 do begin
    for i = 0, n_elements( lons ) - 1 do begin
      plots, lons( i ), lats( j ), psym = 1, symsize = 2.0, color = 0, noclip = 0
    endfor
  endfor
  
  ;- plot longitude labels
  
  n = n_elements( lons_label )
  p = convert_coord( lons_label, rebin( [ latmin ], n ), /data, /to_normal )
  x = reform( p( 0, * ) )
  y = replicate( 0.005, n )
  labels = string( lons_label, format = '( f6.1 )' )
  xyouts, x, y, labels, /normal, color = 0, align = 0.5, charsize = 1.25

  ;- plot latitude labels
  
  n = n_elements( lats_label )
  p = convert_coord( rebin( [ lonmin ], n ), lats_label, /data, /to_normal )
  x = replicate( 0.005, n )
  y = reform( p( 1, * ) )
  labels = string( lats_label, format = '( f5.1 )' )
  xyouts, x, y, labels, /normal, color = 0, align = 0.0, charsize = 1.25

endif

;- update draw window

sharp_plot_update_draw, info, region = region

end
