PRO MAS_NAVIGATE, LAT, LON, HEADING, ALTITUDE, PLAT, PLON

rcs_id = "$Id: mas_navigate.pro,v 1.2 1998/07/24 18:18:22 gumley Exp $"

;+
; Purpose:
;    Compute lat/lon for each pixel in a MAS image given the
;    latitude, longitude, heading, and altitude at nadir.
;
; Usage
;    MAS_NAVIGATE, LAT, LON, HEADING, ALTITUDE, PLAT, PLON
;
; Input:
;    LAT       Array of latitudes at nadir (degrees, -90S to +90N)
;    LON       Array of longitudes at nadir (degrees, -180W to +180E)
;    HEADING   Array of headings (degrees, clockwise from north)
;    ALTITUDE  Array of aircraft altitudes (meters)
;
; Output:
;    PLAT      Array of latitudes for (716,N_ELEMENTS(LAT)) pixels
;    PLON      Array of longitudes for (716,N_ELEMENTS(LAT)) pixels
;
; Revised:
;    10-DEC-1996 Liam Gumley CIMSS/SSEC
;-

;- convert heading to azimuth

ny = n_elements( lat )
azimuth = rebin( rotate( heading, 1 ), 716, ny )
azimuth( 0 : 357, * ) = temporary( azimuth( 0 : 357, * ) ) + 90.0
azimuth( 358 : *, * ) = temporary( azimuth( 358 : *, * ) ) - 90.0
loc = where( azimuth lt 0.0, count )
if count ge 1 then azimuth( loc ) = azimuth( loc ) + 360.0
loc = where( azimuth gt 360.0, count )
if count ge 1 then azimuth( loc ) = azimuth( loc ) -360.0

;- compute distance from nadir to pixel

scan = rebin( abs( ( findgen( 716 ) / 715.0 ) * 85.92 - 0.5 * 85.92 ), 716, ny )
distance = 1.0e-3 * tan( !dtor * scan ) * rebin( rotate( altitude, 1 ), 716, ny )
scan = 0B

;- compute pixel lat/lons

compass, rebin( rotate( lat, 1 ), 716, ny ), rebin( rotate( lon, 1 ), 716, ny ), $
  distance, azimuth, plat, plon, /to_latlon
distance = 0B
azimuth = 0B

END
