PRO MAS_UW_READ, FILE, BAND, IMAGE, DATE, TIME1, TIME2, $
  SCENE = SCENE, CENTER = CENTER, SCANLINE = SCANLINE, NSCANS = NSCANS, $
  LAT = LAT, LON = LON, TIME = TIME

rcs_id = "$Id: mas_uw_read.pro,v 1.2 1998/07/24 18:18:22 gumley Exp $"

;- check scene number

if n_elements( scene ) eq 0 then begin
  record = [ 0, 715 ]
endif else begin
  record = scene
endelse

;- check the band number

if band lt 1 or band gt 50 then begin
  message, 'MAS band number must be in the range [1-50]', /continue
  band = ( 1 > band ) < 50
endif

;- check the file type

type = mas_file_type( file )
if type ne 2 and type ne 4 then $
  message, 'File must be UW Intermediate or Quicklook format'
    
;- open file and get number of complete scans

openr, lun, file, /get_lun
result = fstat( lun )
case type of
  2 : begin
    nscans = long( result.size ) / 74200L
    npixels = 716
    nwords = 742
  end
  4 : begin
    nscans = long( result.size ) / 408L
    npixels = 179
    nwords = 204
  end
endcase

;- check scan number to see if data is byte swapped

swap = 0
test = lonarr( 2 )
readu, lun, test
point_lun, lun, 0L
if test( 1 ) lt 0L or test( 1 ) gt 270000L then swap = 1

;- check whether file is band-interleaved by checking the band number

interleave = 0
data = intarr( nwords, 2 )
readu, lun, data
point_lun, lun, 0L
if swap then data = swap_endian( temporary( data ) )
if data( 15, 0 ) eq data( 15, 1 ) then interleave = 1

;- check record keyword

rec1 = 0
rec2 = nscans - 1
if keyword_set( record ) then begin
  if n_elements( record ) ne 2 then $
    message, 'Keyword RECORD must be an array with 2 elements, e.g. [0,1000]'
  rec1 = ( record( 0 ) > 0 ) < ( nscans - 1 )
  rec2 = ( record( 1 ) > rec1 ) < ( nscans - 1 )
endif

;- create MAS record structure

case type of

  2 : mas_record = { status:0, offset:0, scanline:0L, date:0L, $
        bb1temp:0, bb2temp:0, scanrate:0, hours:0, minutes:0, $
        seconds:0, heading:0, altitude:0, gain:0, band:0, time:0L, $
        bb1count:0, bb2count:0, roll:0, nadirlat:0L, nadirlon:0L, $
        video:intarr( npixels ), headtemp:0 }

        
  4 : mas_record = { status:0, offset:0, scanline:0L, date:0L, $
        bb1temp:0, bb2temp:0, scanrate:0, hours:0, minutes:0, $
        seconds:0, heading:0, altitude:0, gain:0, band:0, time:0L, $
        bb1count:0, bb2count:0, roll:0, nadirlat:0L, nadirlon:0L, $
        video:intarr( npixels ) }

endcase

;- create data array

data = replicate( mas_record, rec2 - rec1 + 1 )

;- read data from disk

case type of

  2 : begin
  
    case interleave of
    
      0 : begin
        scan = replicate( mas_record, 1 )
        n = 0L
        start = rec1 * 74200L + ( band - 1 ) * 2L * 742L
        point_lun, lun, start
        for i = 0, rec2 - rec1 do begin
          readu, lun, scan
          data( n ) = scan
          n = n + 1
          point_lun, lun, start + n * 74200L
        endfor
      end
      
      1 : begin
        point_lun, lun, ( band - 1 ) * 2L * nwords * nscans + ( rec1 * 2L * nwords )
        readu, lun, data
      end
      
    endcase
    
  end
  
  4 : begin
    point_lun, lun, rec1 * 2L * nwords
    readu, lun, data
  end
    
endcase    

;- close data file

free_lun, lun

;- swap data if necessary

if swap then data = swap_endian( temporary( data ) )

;- extract bbc1, bbc2, bbt1, bbt2, headt, scan

bb1c = data.bb1count
bb2c = data.bb2count
bb1t = data.bb1temp * 0.01 + 273.15
bb2t = data.bb2temp * 0.01 + 273.15
if type eq 2 then begin
  headt = data.headtemp * 0.1
endif else begin
  headt = ( bb1t + bb2t ) * 0.5
endelse
scanline = data.scanline

;- get time and date

hour = data.hours
minute = data.minutes
second = data.seconds / 10
date = data( 0 ).date
if date eq 96044587 then date = 96110123
year = date / 1000000L
if year gt 90 then begin
  year = year + 1900L
endif else begin
  year = year + 2000L
endelse
doy = date mod 1000L 
jul2day, doy, year, month, day

;- get navigation data

heading = data.heading / 90.0
altitude = float( data.altitude )
alat = data.nadirlat * 1.0e-5
alon = data.nadirlon * 1.0e-5

;- extract image

image = data.video
data = 0B

;#######################################################################

;- convert image to float array

image = float( temporary( image ) )
  
;- convert date and time variables

date = string( year(0), month(0), day(0), format = '( i4.4,2i2.2 )' )
time1 = string( hour(0), minute(0), second(0), format = '( 3i2.2 )' )
time2 = string( hour(715), minute(715), second(715), format = '( 3i2.2 )' )
time = hour * 10000L + minute * 100L + second

;- get band center values

read_mastbl50, long( date ), iflag, rc, hwn
center = 10000.0 / reform( hwn( 0, * ) )

;- if port 1 or 2, convert to radiance
;- if port 3 or 4, convert to brightness temperature
  
nx = 716
ny = 716

if band le 25 then begin

  ;- convert counts to radiance
   
  image = ( temporary( image ) - iflag( band - 1 ) ) * rc( band - 1 ) * 10.0

  ;- solar spectral irradiances for all bands (taken from ARMCAS HDF)
  
  irradiance = [ 1805.278 , 1529.682 , 1376.341 , 1252.682 , $
    1148.301 , 1046.7 , 946.0285 , 864.8679 , 771.8693 , 234.2306 ,  $
    216.4834 , 190.5309 , 167.6591 , 148.1003 , 133.7146 , 125.9938 , $
    117.4495 , 104.391 , 92.07833 , 82.11004 , 73.59485 , 70.85332 , $
    65.03123 , 56.7776 , 55.43106 , 25.42931 , 21.08938 , 17.88286 , $
    15.49977 , 13.19418 , 11.43108 , 9.96829 , 8.776746 , 7.546451 , $
    6.519591 , 5.647729 , 4.872741 , 4.262437 , 3.758695 , 3.342138 , $
    2.954644 , 0.4262437 , 0.2518713 , 0.1937471 , 0.1549977 , 0.1162483 , $
    0.08718622 , 0.07749885 , 0.0678115 , 0.05812414 ]

  ;- get solar zenith angle
  
  sunzen, alat( 358 ), alon( 358 ), doy( 0 ), $
    hour( 358 ), minute( 358 ), second( 358 ), zenith, azimuth
  
  ;- compute conversion factor for radiance to reflectance
    
  factor = !pi / ( irradiance( band - 1 ) * cos( !dtor * zenith ) )

  ;- convert radiance to reflectance
  
  image = temporary( image ) * factor
  
endif else begin

  ;- blackbody effective emissivity for all bands

  e = [ 1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0, $
    1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0, $
    0.98095, 0.98104, 0.98115, 0.98124, 0.98134, $
    0.98145, 0.98155, 0.98164, 0.98175, 0.98184, $
    0.98194, 0.98203, 0.98213, 0.98223, 0.98233, $
    0.98243, 0.94590, 0.94102, 0.93890, 0.93782, $
    0.93668, 0.93668, 0.93700, 0.93764, 0.93858 ]

  ;- convert bb temps to radiance

  c = 10000.0 / center( band - 1 )
  t1 = bb1t * hwn( 2, band - 1 ) + hwn( 1, band - 1 )
  t2 = bb2t * hwn( 2, band - 1 ) + hwn( 1, band - 1 )
  rad1 = planc_m( c, t1 )
  rad2 = planc_m( c, t2 )

  ;- compute emissivity corrected calibration slope

  emis = e( band - 1 )
  eslope = emis * ( ( rad2 - rad1 ) / ( bb2c - bb1c ) )

  ;- compute emissivity corrected intercept

  h = headt * hwn( 2, band - 1 ) + hwn( 1, band - 1 )
  delrad = planc_m( c, h ) - rad1
  eincpt = rad1 + delrad * ( 1.0 - emis ) - eslope * bb1c

  ;- apply slope and intercept to image data
  
  image = ( temporary( image ) * rebin( rotate( eslope, 1 ), nx, ny, /sample ) ) + $
    rebin( rotate( eincpt, 1 ), nx, ny, /sample )

  ;- convert radiance to brightness temperature
  
  image = brite_m( c, temporary( image ) )

endelse

;- put image in correct orientation (aircraft flying up the display)

image = rotate( rotate( temporary( image ), 7 ), 2 )

;- get lat/lon arrays if requested (set lat and lon to a dummy variable in caller)

if n_elements( lat ) gt 0 or n_elements( lon ) gt 0 then $
  mas_navigate, alat, alon, heading, altitude, lat, lon

END
