;------------------------------------------------------------------------------

pro select_scene_event, event

;- get info structure

widget_control, event.top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- get event user value

widget_control, event.id, get_uvalue = uvalue

;- check user value

case uvalue of

  'Cancel' : begin
    info.cancel = 1
    widget_control, event.top, /destroy
  end

  'OK' : begin

    ;- get time string from text box
    
    widget_control, info.text, get_value = timestring
    
    ;- filter out any non-digits and convert to a long integer
    
    ascii = byte( timestring )
    loc = where( ascii ge 48 and ascii le 57, count )
    if count ge 1 then begin
      time = long( string( ascii( loc ) ) )
    endif else begin
      time = info.time( info.scan1 + 357 )
    endelse
    
    ;- update time value in text box

    time = ( time > info.time( 357 ) ) < info.time( info.nscans - 358 )
    timestring = string( time, format = '(i6.6)' )
    widget_control, info.text, set_value = timestring
    minv = min( abs( time - info.time ), loc )
    scan1 = ( loc - 357 ) > 0
    
    ;- compute new scan range

    info.scan1 = scan1
    info.scan2 = info.scan1 + 716 - 1

    ;- destroy widgets
    
    widget_control, event.top, /destroy
      
  end
  
  'Draw' : begin
  
    ;- check that button was pressed within the draw window
    
    if event.x ge 0 and event.x le ( info.nx - 1 ) and $
       event.y ge 0 and event.y le ( info.ny - 1 ) and event.press then begin
       
      ;- compute new scan range

      info.scan1 = ( ( event.y * info.sub - 357 ) > 0 ) < ( info.nscans - 716 )
      info.scan2 = info.scan1 + 716 - 1
      timestring = string( info.time( info.scan1 + 357 ), format = '(i6.6)' )
      widget_control, info.text, set_value = timestring
      
      ;- erase old box
      
      wset, info.window_id
      device, copy = [ 0, 0, info.nx, info.ny, 0, 0, info.pixmap_id ]
      
      ;- draw new box
      	
	xcoord = [ 0, info.nx - 1, info.nx - 1, 0, 0 ]
	y1 = info.scan1 / info.sub
	y2 = ( info.scan2 / info.sub ) < ( info.ny - 1 )
	ycoord = [ y1, y1, y2, y2, y1 ]
	wset, info.window_id
	plots, xcoord, ycoord, /device, color = 5

    endif

  end
  
  'Text' : begin
  
    ;- get time string from text box
    
    widget_control, info.text, get_value = timestring
    
    ;- filter out any non-digits and convert to a long integer
    
    ascii = byte( timestring )
    loc = where( ascii ge 48 and ascii le 57, count )
    if count ge 1 then begin
      time = long( string( ascii( loc ) ) )
    endif else begin
      time = info.time( info.scan1 + 357 )
    endelse
    
    ;- update time value in text box
    
    time = ( time > info.time( 357 ) ) < info.time( info.nscans - 358 )
    timestring = string( time, format = '(i6.6)' )
    widget_control, info.text, set_value = timestring
    minv = min( abs( time - info.time ), loc )
    scan1 = ( loc - 357 ) > 0
    
    ;- compute new scan range

    info.scan1 = scan1
    info.scan2 = info.scan1 + 716 - 1

    ;- erase old box

    wset, info.window_id
    device, copy = [ 0, 0, info.nx, info.ny, 0, 0, info.pixmap_id ]

    ;- draw new box

    xcoord = [ 0, info.nx - 1, info.nx - 1, 0, 0 ]
    y1 = info.scan1 / info.sub
    y2 = ( info.scan2 / info.sub ) < ( info.ny - 1 )
    ycoord = [ y1, y1, y2, y2, y1 ]
    wset, info.window_id
    plots, xcoord, ycoord, /device, color = 5

    ;- center viewport
    
    widget_control, info.draw, $
      set_draw_view = [ 0, info.scan1/info.sub - info.ysize/2 + info.nx/2 ]
    
  end

  else : print, uvalue
      
end

;- restore info structure

pointer_value, pointer, info, /set, /no_copy

end

;------------------------------------------------------------------------------

PRO SCENE_SELECT, FILE, SCAN1, SCAN2, BAND = BAND, START_RANGE = START_RANGE, $
  GROUP = GROUP, XOFFSET = XOFFSET, YOFFSET = YOFFSET

rcs_id = "$Id: scene_select.pro,v 1.12 1999/06/15 15:13:00 gumley Exp $"

;- check arguments

if n_params() ne 3 then message, 'Syntax: SCENE_SELECT, FILE, SCAN1, SCAN2'
if not keyword_set( band ) then band = 2
if not keyword_set( start_range ) then start_range = [ 0, 715 ]
if n_elements( scan1 ) eq 0 then scan1 = start_range( 0 )
if n_elements( scan2 ) eq 0 then scan2 = start_range( 1 )
if not keyword_set( group ) then group = 0
if not keyword_set( xoffset ) then xoffset = 200
if not keyword_set( yoffset ) then yoffset = 200

;- set constants

npixels = 716
sub = 4
nx = npixels / sub

;- Set display order

!order = 0

;- create widgets

print, '(creating widgets)'
print, 'base 1'
base = widget_base( /column, title = 'Scene Selection', $
  group = group, xoffset = xoffset, yoffset = yoffset, /modal )
device, get_screen_size = sz
ysize = 0.666 * sz( 1 )
print, 'base 2'
base1 = widget_base( base, /column, /frame )
print, 'draw 1'
help, nx, ysize
draw = widget_draw( base1, uvalue = 'Draw', xsize = nx + 75, ysize = 2 * ysize, $
  x_scroll_size = nx + 75, y_scroll_size = ysize )
print, 'base 3'  
text_base = widget_base( base1, /row, /align_center )  
text = cw_field( text_base, title = "Time at center (HHMMSS)", value = ' ', $
  xsize = 8, /return_events, uvalue = 'Text' )
butt_base = widget_base( base, /row, /align_center )
butt1 = widget_button( butt_base, value = 'OK', uvalue = 'OK', xsize = 55 )
butt2 = widget_button( butt_base, value = 'Cancel', uvalue = 'Cancel', xsize = 55 )
widget_control, base, /realize
widget_control, draw, get_value = window_id, set_draw_view = [ 0, 0 ]
wset, window_id
colors
erase, 7
xyouts, ( nx + 50 ) / 2, ( ysize ) / 2 + 30, "Loading quicklook image", $
  /device, color = 0, charsize = 1.5, align = 0.5
xyouts, ( nx + 50 ) / 2, ( ysize ) / 2, "Just a moment....", $
  /device, color = 0, charsize = 1.5, align = 0.5
widget_control, text, sensitive = 0
widget_control, butt1, sensitive = 0
widget_control, butt2, sensitive = 0
widget_control, base, hourglass = 1
print, '(done creating widgets)'

;- open file to read quicklook data

sd_id = hdf_sd_start( file )

;- get image dimensions

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'CalibratedData' ) )
hdf_sd_getinfo, sds_id, dims = dims
hdf_sd_endaccess, sds_id
nscans = long( dims( 2 ) )
ny = nscans / sub

;- get quicklook image (every 4th line and 4th pixel)
;- (Note that the following code paragraph is optimized for speed)

sds_id = hdf_sd_select( sd_id, hdf_sd_nametoindex( sd_id, 'CalibratedData' ) )
start = [ 0, band - 1, 0 ]
count = [ 716, 1, nscans / 8 ]
stride = [ 1, 1, 8 ]
hdf_sd_getdata, sds_id, image, start = start, count = count, stride = stride
hdf_sd_endaccess, sds_id
image = temporary( image ) > 0
image = reform( temporary( image ), 716, nscans / 8 )
image = congrid( temporary( image ), 179, ny )

;- get time code (HHMMSS)

index = hdf_sd_nametoindex( sd_id, 'GpsTime' )
if index lt 0 then index = hdf_sd_nametoindex( sd_id, 'GreenwichMeanTime' )
sds_id = hdf_sd_select( sd_id, index )
hdf_sd_getdata, sds_id, time
hdf_sd_endaccess, sds_id
time = temporary( time ) / 100

;- close file

hdf_sd_end, sd_id
  
;- put image in correct orientation and convert to scaled bytes

image = rotate( rotate( temporary( image ), 7 ), 2 )
minv = min( image, max = maxv )
range = [ minv, maxv ]
result = linearstretch( image, cutoff = 0.001 )
if n_elements( result ) eq 2 then range = result
image = bytscl( temporary( image ), min = range( 0 ), max = range( 1 ), $
  top = !d.table_size - 1 - 16 ) + 16B

;- display image in pixmap (backing store)

window, /free, /pixmap, xsize = nx, ysize = ny
pixmap_id = !d.window
wset, pixmap_id
tv, image

;- resize draw window

widget_control, draw, $
  xsize = nx + 75, draw_xsize = nx + 75, $
  ysize = ( ysize < ny ), draw_ysize = ny
widget_control, draw, set_draw_view = [ 0, 0 ]

;- display image in scrolling window

wset, window_id
erase, 7
tv, image

;- center viewport

widget_control, draw, set_draw_view = [ 0, scan1/sub - ysize/2 + nx/2 ]

;- draw box over first scene

scan1 = start_range( 0 )
scan2 = start_range( 1 )
ycoord = [ scan1, scan1, scan2, scan2, scan1 ] / sub
plots, [ 0, nx - 1, nx - 1, 0, 0 ], ycoord, /device, color = 5

;- draw time markers

loc = uniq( time / 100 ) + 1
n = n_elements( loc )
loc = loc( 0 : n - 2 )
n = n - 1
x = replicate( nx, n )
y = loc / sub
usersym, [ 0, 2.5 ], [ 0, 0 ]
plots, x, y, /device, psym = 8, color = 0
xyouts, x + 10, y - 5, string( time( loc ), format='(i6.6)' ), /device, $
  color = 0, charsize = 1.5

;- turn off hourglass and enable button events in draw window

widget_control, base, hourglass = 0
widget_control, draw, /draw_button_events
timestring = string( time( scan1 + 357 ), format = '(i6.6)' )
widget_control, text, sensitive = 1, set_value = timestring
widget_control, butt1, sensitive = 1
widget_control, butt2, sensitive = 1

;- create info structure

info = { draw      : draw, $
         ysize     : ysize, $
         window_id : window_id, $
         pixmap_id : pixmap_id, $
         nscans    : nscans, $
         time      : time, $
         sub       : sub, $
         scan1     : scan1, $
         scan2     : scan2, $
         nx        : nx, $
         ny        : ny, $
         text      : text, $
         cancel    : 0 }

pointer_create, pointer
pointer_value, pointer, info, /set, /no_copy
widget_control, base, set_uvalue = pointer         

;- start event manager

xmanager, 'select_scene', base

;- retrieve info structure and store results

pointer_value, pointer, info, /no_copy
scan1 = -1
scan2 = -1
if not info.cancel then begin
  scan1 = info.scan1
  scan2 = info.scan2
endif
pointer_free, pointer

END
