PRO SELECT_FONT, HELVETICA = HELVETICA, TIMES = TIMES, $
  PALATINO = PALATINO, COURIER = COURIER, BOLD = BOLD, ITALIC = ITALIC, $
  SIZE = SIZE, NAME = NAME

rcs_id = "$Id: select_font.pro,v 1.3 1998/11/13 23:01:39 gumley Exp $"

;+
; Purpose:
;    Select a Unix hardware font for IDL graphics.
;
; Usage:
;    SELECT_FONT
;
; Input:
;    None required.
;
; Optional Keywords:
;    /HELVETICA    Select a Helvetica font (default)
;    /TIMES        Select a Times font
;    /PALATINO     Select a Palatino font
;    /COURIER      Select a Courier font
;    /BOLD         Select a bold font (default is no bold)
;    /ITALIC       Select and italic font (default is no italics)
;    SIZE          If set to a named variable, sets the font size (default=12)
;    NAME          If set to a named variable, returns the font name selected
;
; Revised:
;    17-OCT-1997 Liam Gumley, CIMSS/SSEC
;    Created
;
; Notes:
;    (1) This procedure currently only works on Unix IDL platforms.
;    (2) The NAME value returned by SELECT_FONT can be used to set the
;        default widget font by the command WIDGET_CONTROL,DEFAULT_FONT=NAME
;
; Example:
; !P.MULTI=[0,1,2,0,0]
; PLOT,INDGEN(10)
; SELECT_FONT,/BOLD
; PLOT,INDGEN(10)
;-

;- this version is only for Unix at the moment

if !version.os_family ne 'unix' then begin
  message, /continue, 'Only works on Unix at the moment'
  return
endif

;- check keyword flags

if not keyword_set( helvetica ) then helvetica = 0
if not keyword_set( times ) then times = 0
if not keyword_set( palatino ) then palatino = 0
if not keyword_set( courier ) then courier = 0
if not keyword_set( bold ) then bold = 0
if not keyword_set( italic ) then italic = 0

;- check keyword values

if n_elements( size ) eq 0 then size = 12

;- set keyword return values

name = ''

;- create font search string

case 1 of
  helvetica : search = '*helvetica*'
  times     : search = '*times*'
  palatino  : search = '*palatino*'
  courier   : search = '*courier*'
  else      : search = '*helvetica*'
endcase

if bold then begin
  search = search + 'bold-'
endif else begin
  search = search + 'medium-'
endelse

if italic then begin
  search = search + 'o-normal*'
endif else begin
  search = search + 'r-normal*'
endelse

;- open a graphics window

current_window = !d.window
window, /free, /pixmap, xsize = 10, ysize = 10

;- get list of font names matching search string

device, font = search, get_fontnames = fontnames

;- find a font size that matches

fontstring = '--' + strcompress( long( size > 8 ), /remove_all ) + '-'
index = strpos( fontnames, fontstring )
loc = where( index ne -1, count )

;- use font if it was found, or else set graphics font size

if count ge 1 then begin
  name = fontnames( loc(0) )
  device, font = name
  !p.font = 0
endif else begin
  message, /continue, 'Requested font was not found - using graphics font instead'
endelse

;- close graphics window

wdelete, !d.window
if current_window gt 0 then wset, current_window

END
