;##############################################################################

PRO SHARP_EVENT, EVENT

@sharp.inc

;- get info structure

widget_control, event.top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- get user value from widget causing event

widget_control, event.id, get_uvalue = uvalue

;- check possible events

case uvalue of

;---------------------------
;--- DESTROY ALL WIDGETS ---
;---------------------------

  'Exit' : widget_control, event.top ,/destroy

;----------------------
;--- FILE SELECTION ---
;----------------------

  'File' : begin

    ;- Desensitize all other widgets
    
    widget_control, event.top, sensitive = 0

    ;- Get filename
    
    widget_control, event.id, tlb_get_offset = offset
    path = dialog_inputfile( path = info.directory, group = event.top, filter = "*.hdf", $
      /must_exist, title = 'MAS HDF File Selection', get_path = directory )

    ;- If file name is good, load the file and update history list
    
    if path ne '' then begin

      ;- Load the new file
      
      sharp_load_file, path, directory, event, info

      ;- Update the history list
      
      result = file_history('.sharp_history.txt', path, /set)

      ;- Destroy the old history menu
      
      for i = 0, n_elements(info.past_ids) - 1 do begin
        if widget_info(info.past_ids[i], /valid) then $
          widget_control, info.past_ids[i], /destroy
      endfor

      ;- Build the new history menu
            
      result = file_history('.sharp_history.txt', files)
      if result then begin
        for i = 0, n_elements(files) - 1 do begin
          info.past_ids[i] = widget_button(info.past, uvalue='Past', value=files[i])
        endfor
      endif

      ;- Sensitize the past files selector if appropriate
      
      if n_elements(files) gt 0 then begin
        widget_control, info.past, sensitive=1
      endif else begin
        widget_control, info.past, sensitive=0
      endelse

    endif
      
    ;- Sensitize all other widgets
    
    widget_control, event.top, sensitive = 1
    
  end
  
;---------------------------------
;--- USER SELECTED A PAST FILE ---
;---------------------------------

  'Past' : begin
  
  ;- Get the file path and name
  
  widget_control, event.id, get_value=path
  
  ;- Separate the directory

  directory = ''
  result = dir_generic(path)
  if n_elements(result) gt 1 then $
    directory = dir_path(result[0:n_elements(result)-2])
    
  ;- If file name is good, load the file and update the history list

  if path ne '' then begin

    widget_control, event.top, sensitive=0

    ;- Load the new file

    sharp_load_file, path, directory, event, info

    ;- Update the history list

    result = file_history('.sharp_history.txt', path, /set)

    ;- Destroy the old history menu

    for i = 0, n_elements(info.past_ids) - 1 do begin
      if widget_info(info.past_ids[i], /valid) then $
        widget_control, info.past_ids[i], /destroy
    endfor

    ;- Build the new history menu

    result = file_history('.sharp_history.txt', files)
    if result then begin
      for i = 0, n_elements(files) - 1 do begin
        info.past_ids[i] = widget_button(info.past, uvalue='Past', value=files[i])
      endfor
    endif

    ;- Sensitize the past files selector if appropriate

    if n_elements(files) gt 0 then begin
      widget_control, info.past, sensitive=1
    endif else begin
      widget_control, info.past, sensitive=0
    endelse

    widget_control, event.top, sensitive=1

  endif
  
  end

;-------------------------
;--- START ZOOM WIDGET ---
;-------------------------

  'Zoom' : begin
    widget_control, info.zoom_button, sensitive = 0
    info.zoom_flag = 1
    pointer_value, pointer, info, /set, /no_copy
    sharp_zoom, event.top, group = event.top
    return
  end

;-------------------------
;--- START PLOT WIDGET ---
;-------------------------

  'Plot' : begin
    widget_control, info.map_button, sensitive = 0
    widget_control, info.plot_button, sensitive = 0
    info.plot_flag = 1
    pointer_value, pointer, info, /set, /no_copy
    sharp_plot, event.top, group = event.top
    return
  end

;--------------------------
;--- START COLOR WIDGET ---
;--------------------------

  'Color' : begin
    widget_control, event.id, tlb_get_offset = offset
    xcolors, group = event.top, bottom = 16, ncolors = !d.table_size - 16, $
      xoffset = offset( 0 ) + 6 * 60, yoffset = offset( 1 ) + 50, $
      file = info.colortable, notifyid = [ info.xcolors_load, event.top ]
  end

;-------------------------
;--- START MASK WIDGET ---
;-------------------------

  'Mask' : begin
    widget_control, info.map_button, sensitive = 0
    widget_control, info.mask_button, sensitive = 0
    info.mask_flag = 1
    pointer_value, pointer, info, /set, /no_copy
    widget_control, event.id, tlb_get_offset = offset
    sharp_mask, event.top, group = event.top, $
      xoffset = offset( 0 ) + 9 * 60, yoffset = offset( 1 ) + 50
    return
  end

;-------------------------
;--- START HIS WIDGET ---
;-------------------------

  'HIS' : begin
    widget_control, info.map_button, sensitive = 0
    widget_control, info.his_button, sensitive = 0
    info.his_flag = 1
    pointer_value, pointer, info, /set, /no_copy
    widget_control, event.id, tlb_get_offset = offset
    sharp_his, event.top, group = event.top, $
      xoffset = offset( 0 ) + 10 * 60, yoffset = offset( 1 ) + 50
    return
  end

;----------------------------
;--- START LOCATE WIDGET ---
;-------------------------

  'Locate' : begin
    widget_control, info.map_button, sensitive = 0
    widget_control, info.locate_button, sensitive = 0
    info.locate_flag = 1
    pointer_value, pointer, info, /set, /no_copy
    widget_control, event.id, tlb_get_offset = offset
    sharp_locate, event.top, group = event.top
    return
  end

;-------------------------------------------------------------------------
;--- UPDATE PIXEL VALUE UNDER CURSOR, AND UPDATE ZOOM WINDOW IF NEEDED ---
;-------------------------------------------------------------------------

  'Draw' : if info.path ne " " then update_pixel_value, event, info

;----------------------
;--- BAND SELECTION ---
;----------------------
  
  'Band' : begin

    widget_control, event.top, sensitive = 0
      
    ;- create band selection dialog
    
    widget_control, event.id, tlb_get_offset = offset
    tlb = widget_base( group = event.top, /column, $
      xoffset = offset( 0 ) + 3 * 60, yoffset = offset( 1 ) + 25, $
      title = 'Band' )
    base = widget_base( tlb , frame = 1 )
    names = strarr( 2, 50 )
    names( 0, * ) = indgen( 50 ) + 1
    names( 1, * ) = info.center
    names = string( names, format='(" Band ",i2.2," (",f5.2,") ",/)' )
    ysize = 20
    if info.nbands eq 12 then begin
      names = strarr( 2, 11 )
      names( 0, * ) = indgen( 11 ) + 2
      names( 1, * ) = info.center( 1 : 11 )
      names = string( names, format='(" Band ",i2.2," (",f5.2,") ",/)' )
      ysize = 11
    endif
    text = widget_list( base, value = names, ysize = ysize, uvalue = 'Band' )
    base = widget_base( tlb, /row, /align_center )
    butt = widget_button( base, value = 'Cancel', uvalue = 'Cancel', xsize = 60 )
    widget_control, tlb, /realize
    
    ;- Get an event (this is a hack that is not recommended!)
    
    new_event = widget_event(tlb, bad_id=bad_id)
    if not widget_info(tlb, /valid_id) then goto, band_select_finish
    widget_control, new_event.id, get_uvalue = uvalue

    ;- load new band if one was selected
    
    if uvalue ne 'Cancel' then begin

      info.band = fix( new_event.index ) + 1
      if info.nbands eq 12 then info.band = info.band + 1
      widget_control, event.top, hourglass = 1
      mas_read, info.path, info.band, image, date, time1, time2, $
	scene = info.scene, units = info.units_flag
      info.image = temporary( image )
      minv = min( info.image, max = maxv )
      info.minv = minv
      info.maxv = maxv
      info.date = date
      info.time1 = time1
      info.time2 = time2
      info.image_flag = 1
      info.formula_flag = 0

      ;- if we were in RGB mode previously, load the saved color table
      
      if info.rgb_flag then tvlct, info.r_table, info.g_table, info.b_table
      info.rgb_flag = 0

      ;- redisplay the image and reset the buttons
      
      mas_display, info, title = info.title
      widget_control, info.color_button, sensitive = 1
      widget_control, info.scale_button, sensitive = 1
	widget_control, event.top, hourglass = 0

    endif

    widget_control, tlb, /destroy

    band_select_finish:
    
    widget_control, event.top, sensitive = 1

  end

;-----------------------
;--- SCENE SELECTION ---
;-----------------------

  'Scene' : begin

    ;- Desensitize top level base

    widget_control, event.top, sensitive=0
    
    ;- Get new scene range
    
    widget_control, event.id, tlb_get_offset = offset
    scan1 = info.scene[0]
    scan2 = info.scene[1]
    band = info.band
    if band eq 99 then band = 2
    scene_select, info.path, scan1, scan2, start_range = [scan1, scan2], $
      group = event.top, band = band, $
      xoffset = offset[0] + 1 * 60, yoffset = offset[1] + 25
    scene = [scan1, scan2]
    
    ;- Load new scene if one was selected
    
    if (scene[0] ne -1) and (scene[0] ne info.scene[0]) then begin
      widget_control, event.top, hourglass=1
      info.scene = scene
      sharp_load_scene, info
      widget_control, event.top, hourglass=0
    endif

    ;- Sensitize top level base

    widget_control, event.top, sensitive=1

  end

;-----------------------
;--- JPG FILE OUTPUT ---
;-----------------------

  'JPG' : begin

    ;- desensitize all other widgets
    
    widget_control, event.top, sensitive = 0
    
    ;- get output file name

    jpg_file = dialog_outputfile( file = info.jpg_file, path = info.jpg_path, $
      filter = '*.jpg', group = event.top, /write, get_path = jpg_path, $
      dialog_parent = info.draw, title = 'Enter JPG filename' )
    if jpg_file eq '' then goto, finish_jpg

    ;- save selected JPG file and path
    
    info.jpg_file = jpg_file
    info.jpg_path = jpg_path
              
    ;- turn on hourglass
    
    widget_control, event.top, hourglass = 1

    ;- decide if this is a true color display
    
    if !d.n_colors gt 256 then true = 1 else true = 0
    
    ;- get image array
    
    wset, info.draw_window
    if true then image = safe_tvrd( true=3 ) else image = safe_tvrd()
    sz = size( image )
    nx = sz( 1 )
    ny = sz( 2 )
    
    ;- get plot array if necessary
    
    plot = [ 0B ]
    plot_nx = 0
    plot_ny = 0
    if info.plot_flag eq 1 and info.region gt 0 then begin
      wset, info.plot_window
      if true then plot = safe_tvrd( true=3 ) else plot = safe_tvrd()
      sz = size( plot )
      plot_nx = sz( 1 )
      plot_ny = sz( 2 )
      loc_white = where( plot eq 7B, count_white )
      loc_black = where( plot eq 0B, count_black )
      if count_white ge 1 then plot( loc_white ) = 0B
      if count_black ge 1 then plot( loc_black ) = 7B
    endif
    
    ;- display the image in a pixmap and then save as JPG 
    
    xsize = nx
    if info.plot_flag then xsize = xsize + plot_nx + 20
    window, /free, /pixmap, xsize = xsize, ysize = ny
    erase, 7
    if true then tv, image, true=3 else $
      tv, image
    if n_elements( plot ) gt 1 then begin
      if true then tv, plot, nx + 20, 0.5 * ( ny - plot_ny ), true=3 else $
        tv, plot, nx + 20, 0.5 * ( ny - plot_ny )
    endif
    xyouts, 0.0, 0.0, "CIMSS/SSEC", /norm, charsize = 1.0, color = 8

    ;- save the image as JPG
    
    saveimage, jpg_file, /jpeg, quality=90

    ;- destroy the pixmap

    wdelete, !d.window

    ;- turn off hourglass
    
    widget_control, event.top, hourglass = 0

    ;- Sensitize widgets when done
    
    finish_jpg:
    widget_control, event.top, sensitive = 1
    
  end

;------------------------------
;--- POSTSCRIPT FILE OUTPUT ---
;------------------------------

  'PS' : begin

    ;- desensitize all other widgets
    
    widget_control, event.top, sensitive = 0
    
    ;- get output file name

    ps_file = dialog_outputfile( file = info.ps_file, path = info.ps_path, $
      filter = '*.ps', group = event.top, /write, get_path = ps_path, $
      dialog_parent = info.draw, title = 'Enter Postscript filename' )
    if ps_file eq '' then goto, finish_ps

    ;- save selected PS file and path
    
    info.ps_file = ps_file
    info.ps_path = ps_path
              
    ;- turn on hourglass
    
    widget_control, event.top, hourglass = 1
    
    ;- save the current display type and font

    current = !d.name
    current_font = !p.font
    
    ;- set up the color table for postscript

    if not info.rgb_flag then begin    

      ;- get the current color table
      
      tvlct, r, g, b, /get
      ncolors = !d.table_size - 16
      bottom = 16
      r_old = r( bottom : bottom + ncolors - 1 )
      g_old = g( bottom : bottom + ncolors - 1 )
      b_old = b( bottom : bottom + ncolors - 1 )
      
      ;- change to Postscript mode
      
      set_plot, 'PS'
      
      ;- load the interpolated color table
      
      ncolors = !d.table_size - 16
      r_new = congrid( r_old, ncolors, /interp )
      g_new = congrid( g_old, ncolors, /interp )
      b_new = congrid( b_old, ncolors, /interp )
      tvlct, r_new, g_new, b_new, bottom
      
    endif
    
    ;- set postscript page size
        
    short_size = 6.5
    if info.plot_flag eq 1 and info.region gt 0 then short_size = 5.0
    short_offset = 0.5 * ( 8.5 - short_size )

    extra = 0.0
    if info.plot_flag eq 1 and info.region gt 0 then extra = 4.0
    long_size = short_size * float( draw_xsize ) / float( draw_ysize ) + extra
    long_offset = 11.0 - 0.5 * ( 11.0 - long_size )

    font_size = 7

    set_plot, 'PS'
    !p.font = 0
    device, /land, /inches, /helv, font_size = font_size, $
	ysize = short_size, xoffset = short_offset, $ 
	xsize = long_size, yoffset = long_offset, $
      file = ps_file, /color, bits = 8

    ;- display image

    region = [ 0.0, 0.0, 1.0, 1.0 ]
    if info.plot_flag eq 1 and info.region gt 0 then $
      region = [ 0.0, 0.0, ( draw_xsize / 716.0 ) * short_size / long_size, 1.0 ]    
    mas_display, info, title = info.title, region = region
    xyouts, 0.0, 0.0, "CIMSS/SSEC", /norm, charsize = 1.0, color = 8

    ;- display plot if necessary
    
    if info.plot_flag eq 1 and info.region gt 0 then begin
      x0 = ( draw_xsize / 716.0 ) * short_size / long_size + 0.025
      x1 = 1.0
      xd = x1 - x0
	scale = 716.0 / draw_ysize
	y0 = scale * 0.1
	y1 = scale * 0.9
      region = [ x0, y0, x1, y1 ]
      sharp_plot_update_plot, info, $
        region = region, $
        xrange = [ info.plot_xmin, info.plot_xmax ], $
        yrange = [ info.plot_ymin, info.plot_ymax ]
    endif      
    
    ;- close postscript file and restore color table, device, and font
    
    device, /close    
    !p.font = -1
    if not info.rgb_flag then tvlct, r_old, g_old, b_old, bottom
    set_plot, current
    !p.font = current_font

    ;- turn off hourglass
    
    widget_control, event.top, hourglass = 0

    ;- Sensitize widgets when done
    
    finish_ps:
    widget_control, event.top, sensitive = 1

  end

;-------------------------
;--- FORMULA SELECTION ---
;-------------------------

  'Math' : begin

    ;- Desensitize all other widgets
    
    widget_control, event.top, sensitive = 0

    ;- Start formula dialog
    
    widget_control, event.id, tlb_get_offset = offset
    bands = info.bands
    operators = info.operators
    formula, info.path, result, group = event.top, title = title, $
      xoffset = offset( 0 ) + 4 * 60, yoffset = offset( 1 ) + 25, $
      bands = bands, operators = operators, $
      scene = info.scene, units = info.units_flag

    if result(0) ne -1.0 then begin

      widget_control, event.top, hourglass = 1

      loc = where( result ne -999.0, count )

      if count eq 0 then begin

        xdisplayfile, " ", text="Sorry, no good formula values were found", $
          done='Done', title='Formula', group=event.top, /modal, $
          width=80, height=4

      endif else begin

        info.image = temporary( result )
	  minv = min( info.image( loc ), max = maxv )
        info.minv = minv
        info.maxv = maxv
        info.band = 99
        info.title = title
        info.bands = bands
        info.operators = operators
        info.image_flag = 0
        info.formula_flag = 1

        ;- if we were in RGB mode previously, load the saved color table

        if info.rgb_flag then tvlct, info.r_table, info.g_table, info.b_table
        info.rgb_flag = 0

        ;- redisplay the image, and reset buttons
        
        mas_display, info, title = info.title
        widget_control, info.color_button, sensitive = 1
        widget_control, info.scale_button, sensitive = 1
        widget_control, event.top, hourglass = 0

      endelse

    endif

    ;- Sensitize all other widgets
    
    widget_control, event.top, sensitive = 1

  end

;-------------------------------
;--- IMAGE SCALING SELECTION ---
;-------------------------------

  'Scale' : begin

    ;- Desensitize all other widgets
    
    widget_control, event.top, sensitive = 0
    
    ;- Create scale selection widgets
      
    widget_control, event.id, tlb_get_offset = offset
    tlb = widget_base( group = event.top, /column, $
      xoffset = offset( 0 ) + 7 * 60, yoffset = offset( 1 ) + 25, $
      title = 'Image Scaling' )

    top_base = widget_base( tlb, /column, /frame, /align_center )
    
    base = widget_base( top_base , /row, /align_center )
    min_id = cw_field( base, value = info.minv, xsize = 6, title = 'Minimum', /float )
    max_id = cw_field( base, value = info.maxv, xsize = 6, title = 'Maximum', /float )
    
    base = widget_base( top_base, /row, /align_center )
    butt = widget_button( base, value = '0% Linear Stretch', $
      uvalue = '0% Linear Stretch', /no_release )
    butt = widget_button( base, value = '2% Linear Stretch', $
      uvalue = '2% Linear Stretch', /no_release )

    base = widget_base( tlb, /row, /align_center )
    butt = widget_button( base, value = 'OK', uvalue = 'OK', xsize = 60 )
    butt = widget_button( base, value = 'Cancel', uvalue = 'Cancel', xsize = 60 )
    widget_control, tlb, /realize

    ;- Get an event (this is a hack that is not recommended!)
    
    scale_get_event:
    new_event = widget_event(tlb, bad_id=bad_id)
    if not widget_info(tlb, /valid_id) then goto, scale_select_finish
    widget_control, new_event.id, get_uvalue = uvalue

    case uvalue of

      '0% Linear Stretch' : begin
        widget_control, event.top, hourglass = 1
        minv = min( info.image, max = maxv )
        widget_control, min_id, set_value = minv
        widget_control, max_id, set_value = maxv
        widget_control, event.top, hourglass = 0
        goto, scale_get_event
      end

      '2% Linear Stretch' : begin
        widget_control, event.top, hourglass = 1
        stretchrange = linearstretch( info.image )
        if n_elements( stretchrange ) eq 2 then begin
          minv = stretchrange( 0 )
          maxv = stretchrange( 1 )
          widget_control, min_id, set_value = minv
          widget_control, max_id, set_value = maxv
        endif
        widget_control, event.top, hourglass = 0
        goto, scale_get_event
      end

      'OK' : begin
        widget_control, min_id, get_value = minv
        widget_control, max_id, get_value = maxv
        if ( minv ne info.minv ) or ( maxv ne info.maxv ) then begin
          if minv ne info.minv then info.minv = minv
          if maxv ne info.maxv then info.maxv = maxv
          widget_control, event.top, hourglass = 1
          mas_display, info, title = info.title
          widget_control, event.top, hourglass = 0
        endif
      end

      else:

    end
    
    ;- Destroy scale selection widgets
    
    widget_control, tlb, /destroy

    scale_select_finish:
    
    ;- Sensitize all other widgets
    
    widget_control, event.top, sensitive = 1

  end

;---------------------------
;--- RGB IMAGE SELECTION ---
;---------------------------

  'RGB' : begin

    ;- Desensitize all other widgets
    
    widget_control, event.top, sensitive = 0
    
    ;- Create RGB band selection widgets
    
    widget_control, event.id, tlb_get_offset = offset
    tlb = widget_base( group = event.top, $
      xoffset = offset( 0 ) + 5 * 60, yoffset = offset( 1 ) + 25, /column, $
      title = 'RGB Composite' )
    top_base = widget_base( tlb, /col, /frame )

    base1 = widget_base( top_base , /row, /align_center )

    base = widget_base( base1, /col, /frame )
    r_id = cw_field( base, value = info.rgb_bands( 0 ), xsize = 3, title = 'Red  ', /int )
    radio_base = widget_base( base, /nonexclusive, /align_center )
    red_radio = widget_button( radio_base, value = 'Grey Flip' )

    base = widget_base( base1, /col, /frame )
    g_id = cw_field( base, value = info.rgb_bands( 1 ), xsize = 3, title = 'Green', /int )
    radio_base = widget_base( base, /nonexclusive, /align_center )
    grn_radio = widget_button( radio_base, value = 'Grey Flip' )

    base = widget_base( base1, /col, /frame )
    b_id = cw_field( base, value = info.rgb_bands( 2 ), xsize = 3, title = 'Blue ', /int )
    radio_base = widget_base( base, /nonexclusive, /align_center )
    blu_radio = widget_button( radio_base, value = 'Grey Flip' )

    base2 = widget_base( top_base, /nonexclusive, /align_center )
    radio = widget_button( base2, value = 'Equalize Histograms' )
    
    base3 = widget_base( tlb, /row, /align_center )
    butt = widget_button( base3, value = 'OK', uvalue = 'OK', xsize = 60 )
    butt = widget_button( base3, value = 'Cancel', uvalue = 'Cancel', xsize = 60 )

    ;- Realize widgets and set buttons
    
    widget_control, tlb, /realize
    widget_control, radio, set_button = info.rgb_equalize
    widget_control, red_radio, set_button = info.rgb_red_flip
    widget_control, grn_radio, set_button = info.rgb_grn_flip
    widget_control, blu_radio, set_button = info.rgb_blu_flip
    
    ;- Get button values
    
    equalize = info.rgb_equalize
    red_flip = info.rgb_red_flip
    grn_flip = info.rgb_grn_flip
    blu_flip = info.rgb_blu_flip
    
    ;- Get an event (this is a hack that is not recommended!)
    
    get_rgb_event:
    new_event = widget_event(tlb, bad_id=bad_id)
    if not widget_info(tlb, /valid_id) then goto, rgb_select_finish
    widget_control, new_event.id, get_uvalue = uvalue
    
    ;- Check for a radio button event

    case new_event.id of
    
      radio : begin
        equalize = equalize xor event.select
        goto, get_rgb_event
      end

      red_radio : begin
        red_flip = red_flip xor event.select
        goto, get_rgb_event
      end
    
      grn_radio : begin
        grn_flip = grn_flip xor event.select
        goto, get_rgb_event
      end

      blu_radio : begin
        blu_flip = blu_flip xor event.select
        goto, get_rgb_event
      end

      else :
    
    endcase
    
    ;- If not radio button, get event value

    widget_control, new_event.id, get_uvalue = uvalue
    
    ;- If OK button was hit, get band values, read new data, display image
    
    if uvalue eq 'OK' then begin

      ;- Turn on hourglass

      widget_control, event.top, hourglass = 1

      ;- Get the RGB band values

      widget_control, r_id, get_value = r_band
      widget_control, g_id, get_value = g_band
      widget_control, b_id, get_value = b_band
      
      lo_band = 1
      hi_band = info.nbands
      if hi_band eq 12 then lo_band = 2
      
      r_band = ( r_band > lo_band ) < hi_band
      g_band = ( g_band > lo_band ) < hi_band
      b_band = ( b_band > lo_band ) < hi_band

      widget_control, r_id, set_value = r_band
      widget_control, g_id, set_value = g_band
      widget_control, b_id, set_value = b_band

      ;- If RGB band values are new, then read new data

      if ( info.rgb_flag eq 0 ) or ( r_band ne info.rgb_bands( 0 ) ) then begin
        mas_read, info.path, r_band, $
          image, date, time1, time2, scene = info.scene, units = info.units_flag
	  info.rgb_image( 0, *, * ) = temporary( image )
      endif

      if ( info.rgb_flag eq 0 ) or ( g_band ne info.rgb_bands( 1 ) ) then begin
        mas_read, info.path, g_band, $
          image, date, time1, time2, scene = info.scene, units = info.units_flag
	  info.rgb_image( 1, *, * ) = temporary( image )
      endif

      if ( info.rgb_flag eq 0 ) or ( b_band ne info.rgb_bands( 2 ) ) then begin
        mas_read, info.path, b_band, $
          image, date, time1, time2, scene = info.scene, units = info.units_flag
	  info.rgb_image( 2, *, * ) = temporary( image )
      endif

      ;- Save the current color table if switching to RGB mode

      if info.rgb_flag eq 0 then begin
        tvlct, r, g, b, /get
        info.r_table = r
        info.g_table = g
        info.b_table = b
      endif

      ;- Update the info structure

      info.rgb_bands = [ r_band, g_band, b_band ]
      info.image_flag = 0
      info.formula_flag = 0
      info.rgb_flag = 1
      info.rgb_equalize = equalize
      info.rgb_red_flip = red_flip
      info.rgb_grn_flip = grn_flip
      info.rgb_blu_flip = blu_flip

      ;- Update the image

      mas_display, info, title = info.title

      ;- Desensitize enhance and scale buttons

      widget_control, info.color_button, sensitive = 0
      widget_control, info.scale_button, sensitive = 0

      ;- Turn off hourglass

      widget_control, event.top, hourglass = 0
        
    endif

    ;- Destroy RGB band selection widget

    widget_control, tlb, /destroy

    rgb_select_finish:
    
    ;- Sensitize all other widgets
    
    widget_control, event.top, sensitive = 1

  end

;----------------------
;--- MAP PROJECTION ---
;----------------------

  'Map' : begin

    ;- desensitize all other widgets
    
    widget_control, event.top, sensitive = 0
    
    ;- create widgets
    
    widget_control, event.id, tlb_get_offset = offset
    tlb = widget_base( group = event.top, $
      xoffset = offset( 0 ) + 10 * 60, yoffset = offset( 1 ) + 25, /column, $
      title = 'Map' )
    top_base = widget_base( tlb, /col, /frame, /align_center )
    base1 = widget_base( top_base, /row, /exclusive, /align_center )
    radio1 = widget_button( base1, value = 'Off', uvalue = 'Off', /no_release )
    radio2 = widget_button( base1, value = 'On', uvalue = 'On', /no_release )
    base2 = widget_base( tlb, /row, /align_center )
    butt = widget_button( base2, value = 'OK', uvalue = 'OK', xsize = 60 )
    butt = widget_button( base2, value = 'Cancel', uvalue = 'Cancel', xsize = 60 )
    widget_control, tlb, /realize
    if info.map_flag then begin
      widget_control, radio2, /set_button
    endif else begin
      widget_control, radio1, /set_button
    endelse
    map_flag = info.map_flag
    
    ;- Get an event (this is a hack that is not recommended!)
    
    get_map_event:
    new_event = widget_event(tlb, bad_id=bad_id)
    if not widget_info(tlb, /valid_id) then goto, map_select_finish
    widget_control, new_event.id, get_uvalue = uvalue

    ;- process radio button events
    
    if uvalue eq 'Off' then begin
      map_flag = 0
      goto, get_map_event
    endif

    if uvalue eq 'On' then begin
      map_flag = 1
      goto, get_map_event
    endif
    
    ;- take action if button value changed and ok button was hit
    
    if uvalue eq 'OK' and map_flag ne info.map_flag then begin
      info.map_flag = map_flag
      widget_control, event.top, hourglass = 1
      mas_display, info, title = info.title
      widget_control, event.top, hourglass = 0
    endif
        
    ;- destroy widgets
    
    widget_control, tlb, /destroy
    
    ;- desensitize plot, mask, and HIS buttons if map projection is turned on
    
    if info.map_flag then begin
      widget_control, info.plot_button, sensitive = 0
      widget_control, info.mask_button, sensitive = 0
      widget_control, info.his_button, sensitive = 0
      widget_control, info.draw, draw_motion_events = 0
      widget_control, info.text, set_value = ''
    endif else begin
      widget_control, info.plot_button, sensitive = 1
      widget_control, info.mask_button, sensitive = 1
      widget_control, info.his_button, sensitive = 1
      widget_control, info.draw, draw_motion_events = 1
    endelse

    map_select_finish:
    
    ;- Sensitize all other widgets
    
    widget_control, event.top, sensitive = 1
    
  end

;-----------------------
;--- UNITS SELECTION ---
;-----------------------

  'Units' : begin

    ;- desensitize all other widgets
    
    widget_control, event.top, sensitive = 0
    
    ;- create widgets
    
    widget_control, event.id, tlb_get_offset = offset
    tlb = widget_base( group = event.top, $
      xoffset = offset( 0 ) + 3 * 60, yoffset = offset( 1 ) + 25, /column, $
      title = 'Units' )
    top_base = widget_base( tlb, /col, /frame, /align_center )
    base1 = widget_base( top_base, /row, /exclusive, /align_center )
    radio1 = widget_button( base1, value = 'Refl/Temp', uvalue = 'Refl/Temp', /no_release )
    radio2 = widget_button( base1, value = 'Radiance', uvalue = 'Radiance', /no_release )
    base2 = widget_base( tlb, /row, /align_center )
    butt = widget_button( base2, value = 'OK', uvalue = 'OK', xsize = 60 )
    butt = widget_button( base2, value = 'Cancel', uvalue = 'Cancel', xsize = 60 )
    widget_control, tlb, /realize
    if info.units_flag then begin
      widget_control, radio2, /set_button
    endif else begin
      widget_control, radio1, /set_button
    endelse
    units_flag = info.units_flag
    
    ;- Get an event (this is a hack that is not recommended!)
    
    get_units_event:
    new_event = widget_event(tlb, bad_id=bad_id)
    if not widget_info(tlb, /valid_id) then goto, units_select_finish
    widget_control, new_event.id, get_uvalue = uvalue

    ;- process radio button events
    
    if uvalue eq 'Refl/Temp' then begin
      units_flag = 0
      goto, get_units_event
    endif

    if uvalue eq 'Radiance' then begin
      units_flag = 1
      goto, get_units_event
    endif
    
    ;- take action if button value changed and ok button was hit
    
    if uvalue eq 'OK' and units_flag ne info.units_flag then begin
    
      ;- turn on hourglass

      widget_control, event.top, hourglass = 1, sensitive = 0

      ;- update units flag

      info.units_flag = units_flag

      ;- read current band

      band = info.band
      if band eq 99 then band = 1
	mas_read, info.path, band, image, date, time1, time2, $
	  scene = info.scene, scanline = scanline, lat = lat, lon = lon, $
	  time = time, units = info.units_flag
	info.scanline = scanline
	info.map_lat = temporary( lat )
	info.map_lon = temporary( lon )

      ;- if the current image is a formula, then compute the formula

      if info.band eq 99 then begin

        formula_compute, info.path, $
          info.bands( 0 ), info.bands( 1 ), info.bands( 2 ), info.bands( 3 ), $
          info.operators( 0 ), info.operators( 1 ), info.operators( 2 ), $
          result, scene = info.scene, units = info.units_flag

        ;-check if formula was computed

	  if result(0) eq -1.0 then begin
          widget_control, tlb, /destroy
          widget_control, event.top, hourglass = 0, sensitive = 1
          goto, finish_units
        endif

        ;- check if computed formula has any good values

        loc = where( result ne -999.0, count )
        if count eq 0 then begin
          xdisplayfile, " ", text="Sorry, no good formula values were found", $
            done='Done', title='Formula', group=event.top, /modal, $
            width=80, height=4
          widget_control, event.top, hourglass = 0, sensitive = 1
          goto, finish_units
        endif

        image = temporary( result )

      endif
        
      ;- update info structure items                  

      info.image = temporary( image )
      minv = min( info.image, max = maxv )
      info.minv = minv
      info.maxv = maxv
      info.date = date
      info.time1 = time1
      info.time2 = time2
      info.time = temporary( time )
        
      ;- if rgb image, read rgb data

      if info.rgb_flag then begin

        mas_read, info.path, info.rgb_bands( 0 ), $
          image, date, time1, time2, scene = info.scene, units = info.units_flag
	  info.rgb_image( 0, *, * ) = temporary( image )

        mas_read, info.path, info.rgb_bands( 1 ), $
          image, date, time1, time2, scene = info.scene, units = info.units_flag
	  info.rgb_image( 1, *, * ) = temporary( image )

        mas_read, info.path, info.rgb_bands( 2 ), $
          image, date, time1, time2, scene = info.scene, units = info.units_flag
	  info.rgb_image( 2, *, * ) = temporary( image )

      endif

      ;- if cloud mask flag is set, then update cloud mask

      if info.mask_flag then begin

        ;- read the cloud mask data

        mask_read, info.mask_path, info.date, info.start_time, info.nscans, mask, $
          scene = info.scene

        if mask( 0 ) ne -1 then begin

          ;- update info structure

          info.mask_data = temporary( mask )

        endif

      endif
	  
	;- display current image

      mas_display, info, title = info.title

      ;- update plot if it is defined

	if info.plot_flag then begin
        widget_control, info.plot_button, sensitive = 0
        info.region = 0
        info.plot_xdata( 0 ) = -999.9
        info.plot_ydata( 0 ) = -999.9
	  for i = 0, 9 do begin
	    pointer_value, info.region_ptr( i ), region_data, /no_copy
	    region_data = { result: -1 }
	    pointer_value, info.region_ptr( i ), region_data, /set, /no_copy
	  endfor
        sharp_plot_update_draw, info
        sharp_plot_update_plot, info
	endif

    endif

    finish_units:
    
    ;- destroy widgets
    
    widget_control, tlb, /destroy

    units_select_finish:
    
    ;- Sensitize all other widgets
    
    widget_control, event.top, hourglass = 0, sensitive = 1
    
  end

;---------------------------------------
;--- XCOLORS CHANGED THE COLOR TABLE ---
;---------------------------------------

  'xcolors_load' : begin

    ;- Save the current window id
      
    current_window = !d.window

    ;- Reset the window id
    
    if current_window ge 0 then wset, current_window
        
  end

;-----------------------------------------------------
;--- USER REQUESTED PROGRAM INFORMATION AT STARTUP ---
;-----------------------------------------------------

  'Hidden' : begin

    ;- Create GPL string array
    
    text = gpl( 'Viewer for MODIS Airborne Simulator data', 1999, 'Liam E. Gumley' )
    
    ;- Display the message
    
    xdisplayfile, " ", text=text, done='Done', title='About this program', $
      group=event.top, /modal
    
  end

;--------------------------
;--- UNIDENTIFIED EVENT ---
;--------------------------

  else: print, 'UNRECOGNIZED EVENT, UVALUE = ', uvalue
  
endcase

;- point back to startup window

wset, info.startup_window

;- update info structure

pointer_value, pointer, info, /set, /no_copy

end

;##############################################################################

PRO SHARP_EXIT, EVENT

;- Kill the widget (which causes the cleanup routine to be called)

widget_control, event.top, map = 0
widget_control, event.top, /destroy

END

;##############################################################################

PRO SHARP_CLEANUP, ID

;- Get info structure

widget_control, id, get_uvalue = pointer

;- Free pointers

pointer_value, pointer, info
pointer_free, info.region_ptr
pointer_free, pointer

END

;##############################################################################

PRO SHARP, COLORTABLE = COLORTABLE

rcsid = "$Id: sharp.pro,v 1.46 1999/11/16 18:03:24 gumley Exp $"

@sharp.inc

;- Check keywords

if n_elements( colortable ) eq 0 then $
  colortable = filepath( subdir = [ 'resource', 'colors'], 'colors1.tbl' )

;- Initialize the color system

colorset, /quiet
startup_window = !d.window

;- Check that this is a supported display

device, get_visual_name=visual_name
if (visual_name ne 'PseudoColor') and (visual_name ne 'TrueColor' ) then begin
  message, 'This application requires a PseudoColor or TrueColor display', /continue
  message, 'Please exit IDL, and restart SHARP before any other IDL programs'
endif  

;- Check that sufficient colors are available

if !d.table_size lt 32 then begin
  message, "At least 32 colors are required, and only " + $
    strcompress( !d.table_size ) + " were found.", /continue
  message, "Please exit IDL, close all other applications, and then restart IDL and SHARP."
endif

;- Check that we are running under IDL version 5 or higher

if long(!version.release) lt 5 then message, 'IDL Version 5 or higher is required'

;- set font sizes

select_font, size = 12, name = name
if n_elements( name ) gt 0 then widget_control, default_font = name
select_font, size = 14, /bold

;- create top level base widget

tlb = widget_base( tlb_frame_attr = 1, /column, $
  uvalue = 'TLB', title = 'MAS Scene Viewer' )
infobase = tlb
widget_control, tlb, /managed

;- create widget which will be notified by XCOLORS.PRO when color table changes

xcolors_load = widget_base( tlb, uvalue = 'xcolors_load' )

;- create buttons

butt_size = 49
base = widget_base( tlb, /row, /frame, title = 'MAS Scene Viewer' )
menu   = widget_button( base, value = 'File', /menu, xsize= butt_size )
butt0  = widget_button( menu, value = 'Open...', uvalue = 'File' )
past   = widget_button( menu, value = 'Previous', /menu )
butt1  = widget_button( base, value = 'Scene', uvalue = 'Scene', xsize= butt_size )
butt2  = widget_button( base, value = 'Band', uvalue = 'Band', xsize= butt_size )
butt3  = widget_button( base, value = 'Units', uvalue = 'Units', xsize= butt_size )
butt4  = widget_button( base, value = 'Math', uvalue = 'Math', xsize= butt_size )
butt5  = widget_button( base, value = 'RGB', uvalue = 'RGB', xsize= butt_size )
butt6  = widget_button( base, value = 'Color', uvalue = 'Color', xsize= butt_size )
butt7  = widget_button( base, value = 'Scale', uvalue = 'Scale', xsize= butt_size )
butt8  = widget_button( base, value = 'Plot', uvalue = 'Plot', xsize= butt_size )
butt9  = widget_button( base, value = 'Mask', uvalue = 'Mask', xsize= butt_size )
butt10 = widget_button( base, value = 'Map', uvalue = 'Map', xsize= butt_size )
;***
;***butt14 = widget_button( base, value = 'Locate', uvalue = 'Locate', xsize= butt_size )
;***
zoom_button = widget_button( base, value = 'Zoom', uvalue = 'Zoom', xsize= butt_size )
butt11 = widget_button( base, value = 'HIS', uvalue = 'HIS', xsize= butt_size )
butt12 = widget_button( base, value = 'JPG', uvalue = 'JPG', xsize= butt_size )
butt13 = widget_button( base, value = 'PS', uvalue = 'PS', xsize= butt_size )
exit_button = widget_button( menu, value = 'Exit', uvalue = 'Exit', $
  event_pro='sharp_exit', /separator )
;***buttons = [butt0, butt1, butt2, butt3, butt4, butt5, butt6, butt7, butt8, $
;***  butt9, butt10, butt11, butt12, butt13, butt14]
buttons = [butt0, butt1, butt2, butt3, butt4, butt5, butt6, butt7, butt8, $
  butt9, butt10, butt11, butt12, butt13]

;- set contents of historical file list

past_ids = lonarr(5)
result = file_history('.sharp_history.txt', files)
if result then begin
  for i = 0, n_elements(files) - 1 do begin
    past_ids[i] = widget_button(past, uvalue='Past', value=files[i])
  endfor
endif
if n_elements(files) gt 0 then begin
  widget_control, past, sensitive=1
endif else begin
  widget_control, past, sensitive=0
endelse

;- create draw widget

device, get_screen_size = sz
xsize = draw_xsize
ysize = draw_ysize
scroll = 0
if sz(1) lt 1024 then begin
  ysize = sz(1) - 150
  scroll = 1
endif
base = widget_base( tlb )
draw = widget_draw( base, scroll = scroll, uvalue = 'Draw', /button_events )
widget_control, draw, xsize = xsize, ysize = ysize, $
  draw_xsize = draw_xsize, draw_ysize = draw_ysize

;- create hidden text widget

hidden = widget_text( base, scr_xsize = 1, scr_ysize = 1, /all, frame = 0, $
  uvalue = 'Hidden' )

;- create text display

base = widget_base( tlb, /row )
text = widget_label( base, value = ' ', xsize = draw_xsize - 60, /align_left )

;- desensitize all buttons

for i = 1, n_elements( buttons ) - 1 do begin
  widget_control, buttons( i ), sensitive = 0
endfor
widget_control, zoom_button, sensitive = 0

;- realize widgets

widget_control, tlb, /realize
widget_control, draw, get_value = draw_window

;- load graphics colors and image greyscale lookup table

colors
loadct, 0, bottom = 16, ncolors = !d.table_size - 16, file = colortable
tvlct, r_table, g_table, b_table, /get

;- create pixmaps of the same size as the draw widget (for backing store)

window, /free, /pixmap, xsize = draw_xsize, ysize = draw_ysize
draw_pixmap = !d.window
window, /free, /pixmap, xsize = draw_xsize, ysize = draw_ysize
mask_pixmap = !d.window
window, /free, /pixmap, xsize = draw_xsize, ysize = draw_ysize
mask_pixmap1 = !d.window

;- save display order

default_order = !order

;- display startup message

versioninfo = str_sep( rcsid, " " )
version = "Version " + versioninfo( 2 ) + ", " + versioninfo( 3 )

select_font, size = 24, /italic
wset, draw_window
xyouts, 0.5, 0.65, '!17MAS Scene Viewer!X', $
  /normal, align = 0.5, charsize = 4.0, color = 7
xyouts, 0.5, 0.55, '!17Liam.Gumley@ssec.wisc.edu!X!C!17' + version + '!X', $
  /normal, align = 0.5, charsize = 2.5, color = 7
xyouts, 0.5, 0.40, '!17(For information about this program, hit any key)!X', $
  /normal, align = 0.5, charsize = 2.5, color = 7
xyouts, 0.5, 0.30, '!17Select a MAS HDF file to begin!X', $
  /normal, align = 0.5, charsize = 2.5, color = 4
select_font, size = 14, /bold

;- print a warning if less than 48 colors found (16 for graphics and 32 for images)

if !d.table_size lt 48 then begin
  warning = 'Warning: SHARP found only ' + strcompress( !d.table_size ) + ' colors.'
  xyouts, 0.5, 0.15, warning, /normal, align = 0.5, charsize = 4.0, color = 5
  warning = "For best results, please exit IDL, close all other applications, and then restart IDL and SHARP."
  xyouts, 0.5, 0.10, warning, /normal, align = 0.5, charsize = 4.0, color = 5
endif  

;- Set input focus to keyboard in case user requests program information

widget_control, hidden, /INPUT_FOCUS  

;- Create info structure

pointer_create, region_ptr, npointers=10
for i = 0, 9 do begin
  pointer_value, region_ptr(i), {result:-1}, /set
endfor

info = { main_base      : tlb, $
         path           : " ", $
         directory      : ".", $
         nscans         : 0L, $
         scene          : [ 0, 715 ], $
         band           : 0, $
         nbands         : 0, $
         center         : fltarr( 50 ), $
         image          : fltarr( 716, 716 ), $
         minv           : 0.0, $
         maxv           : 0.0, $
         date           : " ", $
         start_time     : " ", $
         time1          : " ", $
         time2          : " ", $
         time           : lonarr( 716 ), $
         title          : " ", $
         expname        : " ", $
         track_number   : 0L, $
         scanline       : lonarr( 716 ), $
         image_flag     : 0, $
         formula_flag   : 0, $
         units_flag     : 0, $
         colortable     : colortable, $
         r_table        : r_table, $
         g_table        : g_table, $
         b_table        : b_table, $
         rgb_flag       : 0, $
         rgb_bands      : [ 10, 9, 2 ], $
         rgb_image      : fltarr( 3, 716, 716 ), $
         rgb_equalize   : 0, $
         rgb_red_flip   : 0, $
         rgb_grn_flip   : 0, $
         rgb_blu_flip   : 0, $
         bands          : [ 1, 2, 3, 4 ], $
         operators      : [ 0, 0, 0 ], $
         draw           : draw, $
         draw_window    : draw_window, $
         draw_pixmap    : draw_pixmap, $
         text           : text, $
         buttons        : buttons, $
         plot_button    : butt8, $
         plot_flag      : 0, $
         plot_base      : -1L, $
         plot_drawid    : -1L, $
         plot_window    : -1L, $
         clear_id       : -1L, $
         axes_id        : -1L, $
         save_id        : -1L, $
         region         : 0, $
         region_ptr     : region_ptr, $
         x_bands        : [ 1, 2, 3, 4 ], $
         x_operators    : [ 0, 0, 0 ], $
         x_title        : " ", $
         plot_xdata     : replicate( -999.9, 716, 716 ), $
         plot_xmin      : 1.0e20, $
         plot_xmax      : -1.0e20, $
         x_min_id       : -1L, $
         x_max_id       : -1L, $
         x_label_id     : -1L, $
         y_bands        : [ 1, 2, 3, 4 ], $
         y_operators    : [ 0, 0, 0 ], $
         y_title        : " ", $
         plot_ydata     : replicate( -999.9, 716, 716 ), $
         plot_ymin      : 1.0e20, $
         plot_ymax      : -1.0e20, $
         y_min_id       : -1L, $
         y_max_id       : -1L, $
         y_label_id     : -1L, $
         save_flag      : 0, $
         mask_button    : butt9, $
         mask_flag      : 0, $
         mask_base      : -1L, $
         mask_topbase   : -1L, $
         mask_botbase   : -1L, $
         mask_path      : " ", $
         mask_data      : bytarr( 6, 716, 716 ), $
         mask_names     : [ 'Confident Clear   ', $
                            'Probably Clear    ', $
                            'Uncertain         ', $
                            'Cloudy	       ' ], $
         mask_select    : [ 1L, 0L, 0L, 0L ], $
         mask_test      : [ 5, 4, 3, 2 ], $
         mask_color     : [ 4, 2, 5, 7 ], $
         mask_buttons   : [ -1L, -1L, -1L, -1L ], $
         mask_list      : 25, $
         mask_pixmap    : mask_pixmap, $
         mask_pixmap1   : mask_pixmap1, $
         mask_toggle    : 1L, $
         mask_toggle1   : 0L, $
         map_button     : butt10, $
         map_flag       : 0, $
         map_lat        : fltarr( 716, 716 ), $
         map_lon        : fltarr( 716, 716 ), $
         solzen         : fltarr( 716, 716 ), $
         solazm         : fltarr( 716, 716 ), $
         senzen         : fltarr( 716, 716 ), $
         senazm         : fltarr( 716, 716 ), $
         heading        : fltarr( 716 ), $
         his_button     : butt11, $
         his_flag       : 0, $
         his_base       : -1L, $
         his_label      : -1L, $
         his_path       : " ", $
         his_directory  : ".", $
         his_number     : 0L, $
         his_time       : lonarr( 10000 ), $
         zoom_flag      : 0, $
         zoom_base      : -1L, $
         zoom_button    : zoom_button, $
         zoom_window    : -1, $
         zoom_xsize     : 300, $
         zoom_ysize     : 300, $
         zoom_factor    : 3, $
         xcolors_load   : xcolors_load, $
         jpg_file       : 'sharp.jpg', $
         jpg_path       : '', $
         ps_file        : 'sharp.ps', $
         ps_path        : '', $
         plot_file      : 'plot.dat', $
         plot_path      : '', $
         past           : past, $
         past_ids       : past_ids, $
         startup_window : startup_window, $
         color_button   : butt6, $
         scale_button   : butt7, $
         jpg_button     : butt12, $
         ps_button      : butt13, $
;***
;***         locate_button  : butt14, $
;***
         locate_flag    : 0, $
         locate_path    : " ", $
         locate_directory : "." }

;- Store info structure
         
pointer_create, pointer
pointer_value, pointer, info, /set, /no_copy
widget_control, infobase, set_uvalue = pointer

;- Point to startup window

wset, startup_window

;- manage events

xmanager, /catch
xmanager, "sharp", tlb, cleanup='sharp_cleanup', /no_block

;- restore display order

wset, startup_window
!order = default_order

end

;##############################################################################
