PRO SHARP_LOAD_FILE, PATH, DIRECTORY, EVENT, INFO

;- Load and display a new file after a file is selected

;- get file type

widget_control, event.top, hourglass = 1
filetype = mas_file_type( path, nscans = nscans )
widget_control, event.top, hourglass = 0

;- check for unrecognized file,
;- or recognized MAS HDF file with too few scans

status = 0

if filetype ne 3 then begin
  status = 1
  string = "Sorry, " + path + " is not a valid MAS HDF file"
endif

if filetype eq 3 and nscans lt 716 then begin
  status = 1
  string = "Sorry, " + path + " contains " + $
    strcompress( nscans, /remove_all ) + $
    " scans" + string(10B) + $
    "Please choose a MAS HDF file with 716 or more scans"
endif

;- if file is no good, create widget message

if status then begin
  xdisplayfile, " ", text=string, $
    done='Done', title='File', group=event.top, /modal, $
    width=80, height=4
  goto, finish_file
endif

;- file looks good, so let's read it
;- turn on hourglass, and add filename to widget title

widget_control, event.top, hourglass = 1, $
  tlb_set_title = 'MAS Scene Viewer: ' + path

;- read the new file

band = 2
scene = [ 0, 715 ]
mas_read, path, band, image, date, time1, time2, $
  scene = scene, center = center, scanline = scanline, $
  nscans = nscans, lat = lat, lon = lon, time = time, units = info.units_flag, $
  solzen = solzen, solazm = solazm, senzen = senzen, senazm = senazm, $
  expname = expname, track = track_number, heading = heading

;- update the info structure

info.path = path
info.directory = directory
info.scene = scene
info.band = band
info.nscans = nscans
info.image = temporary( image )
minv = min( info.image, max = maxv )
info.minv = minv
info.maxv = maxv
info.date = date
info.start_time = time1
info.time1 = time1
info.time2 = time2
info.time = temporary( time )
info.nbands = n_elements( center )
info.center( 0 : info.nbands - 1 ) = center
info.scanline = scanline
info.image_flag = 1
info.formula_flag = 0
info.map_lat = temporary( lat )
info.map_lon = temporary( lon )
info.solzen  = temporary( solzen )
info.solazm  = temporary( solazm )
info.senzen  = temporary( senzen )
info.senazm  = temporary( senazm )
info.expname = expname
info.track_number = track_number
info.heading = temporary( heading )

;- if we were in RGB mode previously, load the saved color table

if info.rgb_flag then begin
  wset, info.draw_window
  erase, 7
  tvlct, info.r_table, info.g_table, info.b_table
endif
info.rgb_flag = 0

;- turn mask off

if info.mask_flag then widget_control, info.mask_base, /destroy
info.mask_flag = 0
info.mask_path = " "

;- turn HIS off

if info.his_flag then widget_control, info.his_base, /destroy
info.his_flag = 0
info.his_path = " "

;- display the new image

mas_display, info, title = info.title

;- turn on motion events in draw window

widget_control, info.draw, draw_motion_events = 1

;- sensitize buttons

for i = 1, n_elements( info.buttons ) - 1 do begin
  widget_control, info.buttons( i ), sensitive = 1
endfor
if check_demo_mode() then begin
  widget_control, info.jpg_button, sensitive = 0
  widget_control, info.ps_button,  sensitive = 0
endif  
if info.zoom_flag eq 0 then widget_control, info.zoom_button, sensitive = 1

;- if plot window was open, clear all plot regions

if info.plot_flag then begin
  widget_control, info.plot_button, sensitive = 0
  info.region = 0
  info.plot_xdata( 0 ) = -999.9
  info.plot_ydata( 0 ) = -999.9
  for i = 0, 9 do begin
    pointer_value, info.region_ptr( i ), region_data, /no_copy
    region_data = { result: -1 }
    pointer_value, info.region_ptr( i ), region_data, /set, /no_copy
  endfor
  sharp_plot_update_draw, info
  sharp_plot_update_plot, info
endif

;- turn off hourglass

finish_file:
widget_control, event.top, hourglass = 0

END
