PRO SHARP_LOAD_SCENE, INFO

;- Load a new scene

;- Read current band and update info structure

band = info.band
if band eq 99 then band = 1
mas_read, info.path, band, image, date, time1, time2, $
  scene = info.scene, scanline = scanline, lat = lat, lon = lon, $
  time = time, units = info.units_flag, $
solzen = solzen, solazm = solazm, senzen = senzen, senazm = senazm, $
heading = heading
info.scanline = scanline
info.map_lat  = temporary( lat )
info.map_lon  = temporary( lon )
info.solzen   = temporary( solzen )
info.solazm   = temporary( solazm )
info.senzen   = temporary( senzen )
info.senazm   = temporary( senazm )
info.heading  = temporary( heading )
info.date     = date
info.time1    = time1
info.time2    = time2
info.time     = temporary( time )

;- If the current image is a formula, then compute the formula

if info.band eq 99 then begin

  ;- Compute formula

  formula_compute, info.path, $
    info.bands( 0 ), info.bands( 1 ), info.bands( 2 ), info.bands( 3 ), $
    info.operators( 0 ), info.operators( 1 ), info.operators( 2 ), $
    result, scene = info.scene, units = info.units_flag

  ;- Check if formula computation was successful

  if result(0) eq -1.0 then begin
    widget_control, tlb, /destroy
    widget_control, event.top, hourglass = 0, sensitive = 1
    goto, finish_scene
  endif

  ;- Check if computed formula has any good values

  loc = where( result ne -999.0, count )
  if count lt 1 then begin
    xdisplayfile, " ", text="Sorry, no good formula values were found", $
      done='Done', title='Formula', group=event.top, /modal, $
      width=80, height=4
    widget_control, event.top, hourglass = 0, sensitive = 1
    goto, finish_scene
  endif

  image = temporary( result )

endif

;- Update image information in info structure

info.image = temporary( image )
minv = min( info.image, max = maxv )
info.minv = minv
info.maxv = maxv

;- If rgb image, read rgb data

if info.rgb_flag then begin

  mas_read, info.path, info.rgb_bands( 0 ), $
    image, date, time1, time2, scene = info.scene, units = info.units_flag
    info.rgb_image( 0, *, * ) = temporary( image )

  mas_read, info.path, info.rgb_bands( 1 ), $
    image, date, time1, time2, scene = info.scene, units = info.units_flag
    info.rgb_image( 1, *, * ) = temporary( image )

  mas_read, info.path, info.rgb_bands( 2 ), $
    image, date, time1, time2, scene = info.scene, units = info.units_flag
    info.rgb_image( 2, *, * ) = temporary( image )

endif

;- If cloud mask file has been opened, then update cloud mask

if info.mask_path ne " " then begin
  mask_read, info.mask_path, info.date, info.start_time, info.nscans, mask, $
    scene = info.scene
  if mask( 0 ) ne -1 then info.mask_data = temporary( mask )
endif

;- Display current image

mas_display, info, title = info.title

;- Delete all plot regions

info.region = 0
info.plot_xdata( 0 ) = -999.9
info.plot_ydata( 0 ) = -999.9

for i = 0, 9 do begin
  pointer_value, info.region_ptr( i ), region_data, /no_copy
  region_data = { result: -1 }
  pointer_value, info.region_ptr( i ), region_data, /set, /no_copy
endfor

;- Update plot if it exists

if info.plot_flag then begin
  sharp_plot_update_draw, info
  sharp_plot_update_plot, info
endif

finish_scene:
    
END
