;##############################################################################

PRO SHARP_MASK_EVENT, EVENT

@sharp.inc

;- get info structure

widget_control, event.top, get_uvalue = top
widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- get user value from widget causing event

widget_control, event.id, get_uvalue = uvalue

;- check possible events

if n_elements( uvalue ) eq 1 then case uvalue of

  'TLB' : if event.enter eq 1 then begin
    main_base = info.main_base
    pointer_value, pointer, info, /set, /no_copy
    widget_control, main_base, event_pro = 'sharp_event'
    return
  end

  'PLOT_TLB' : if event.enter eq 1 then begin
    plot_base = info.plot_base
    pointer_value, pointer, info, /set, /no_copy
    widget_control, plot_base, event_pro = 'sharp_plot_event'
    return
  end

  'MASK_TLB' : if event.enter eq 1 then begin
    mask_base = info.mask_base
    pointer_value, pointer, info, /set, /no_copy
    widget_control, mask_base, event_pro = 'sharp_mask_event'
    return
  end

  'File' : begin

    path = dialog_inputfile( path = info.directory, group = event.top, filter = "*.cm", $
      /must_exist, title = 'Cloud Mask File Selection' )

    if path ne '' then begin
    
      widget_control, event.top, hourglass = 1
      
      ;- read the cloud mask data
      
      mask_read, path, info.date, info.start_time, info.nscans, mask, $
        scene = info.scene

      if mask( 0 ) ne -1 then begin

        ;- update info structure
              
        info.mask_flag = 1
        info.mask_path = path
        info.mask_data = temporary( mask )
        
        ;- display the image
        
        mas_display, info, title = info.title

        ;- sensitize buttons
        
        widget_control, info.mask_topbase, sensitive = 1
        widget_control, info.mask_botbase, sensitive = 1
        
      endif
      
      widget_control, event.top, hourglass = 0
      
    endif    
    
  end

  'Toggle' : begin
    info.mask_toggle = 1 xor info.mask_toggle
    sharp_plot_update_draw, info
  end

  'Test Toggle' : begin
    info.mask_toggle1 = 1 xor info.mask_toggle1
    if ( not info.mask_toggle ) then begin
      info.mask_toggle = 1
      info.mask_toggle1 = 1
    endif
    sharp_plot_update_draw, info
  end

  'Apply' : begin
    widget_control, top, hourglass = 1, sensitive = 0
    info.mask_toggle = 1
    mas_display, info, title = info.title, /mask_only
    sharp_plot_update_draw, info
    widget_control, top, hourglass = 0, sensitive = 1
  end

  ;- convert droplist selection to test number used in mask_image.pro
  ;- (note offset value of 6 applied to event.index)
  
  'Test List' : begin
    widget_control, top, hourglass = 1, sensitive = 0
    info.mask_list = event.index + 6
    info.mask_toggle = 1
    info.mask_toggle1 = 1
    mas_display, info, title = info.title, /mask_only
    sharp_plot_update_draw, info
    widget_control, top, hourglass = 0, sensitive = 1
  end

  else : print, uvalue
    
end

;- check radio button events

loc = where( event.id eq info.mask_buttons, count )
if count eq 1 then begin
  info.mask_select( loc ) = event.select
endif

;- update info structure

pointer_value, pointer, info, /set, /no_copy

end

;##############################################################################

PRO SHARP_MASK_DISMISS, EVENT

;- Kill the widget (which causes the cleanup routine to be called)

widget_control, event.top, map = 0
widget_control, event.top, /destroy

END

;##############################################################################

PRO SHARP_MASK_CLEANUP, ID

;- Get id of top level widget, and return if invalid
;- (this happens when the top level widget is destroyed)

widget_control, id, get_uvalue = top
if not widget_info( top, /valid_id ) then return

;- Get info structure

widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- Set mask off

info.mask_flag = 0
widget_control, top, hourglass = 1
mas_display, info, title = info.title
widget_control, top, hourglass = 0
widget_control, info.mask_button, sensitive = 1
if ( not info.plot_flag ) and ( not info.his_flag ) $
  then widget_control, info.map_button, sensitive = 1

;- Update info structure

pointer_value, pointer, info, /set, /no_copy

END

;##############################################################################

PRO SHARP_MASK, TOP, GROUP = GROUP, XOFFSET = XOFFSET, YOFFSET = YOFFSET

rcsid = "$Id: sharp_mask.pro,v 1.11 1999/06/10 18:57:46 gumley Exp $"

@sharp.inc

;- get info structure

widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- check keywords

if not keyword_set( group ) then group = top
device, get_screen_size = sz
if not keyword_set( xoffset ) then xoffset = sz( 0 ) / 2
if not keyword_set( yoffset ) then yoffset = sz( 1 ) / 2

;- create base widget

tlb = widget_base( group = group, column = 1, $
  uvalue = 'MASK_TLB', title = 'Cloud Mask', $
  xoffset = xoffset, yoffset = yoffset )
infobase = tlb

;- create menu widgets

b = widget_base( tlb, /row, /frame )
butt = widget_button( b, value = 'File', uvalue = 'File', xsize = 55 )
butt = widget_button( b, value = 'Dismiss', uvalue = 'Dismiss', xsize = 55, $
  event_pro = 'sharp_mask_dismiss' )

;- create cloud mask result widgets

top_base = widget_base( tlb, /col, /frame )
label = widget_label( top_base, value = 'Result Classes' )
draw_ids = lonarr( n_elements( info.mask_names ) )
for i = 0, n_elements( info.mask_names ) - 1 do begin
  base = widget_base( top_base, /row, /base_align_center )
  b = widget_base( base, /nonexclusive, xsize = 135 )
  info.mask_buttons( i ) = widget_button( b, value = info.mask_names( i ) )
  draw_ids( i ) = widget_draw( base, xs = 20, ys = 20 )
endfor
b = widget_base( top_base, /row, /align_center )
butt = widget_button( b, value = 'Apply', uvalue = 'Apply', xsize = 65 )
butt = widget_button( b, value = 'Toggle', uvalue = 'Toggle', xsize = 65 )

;- create cloud mask test widgets

bot_base = widget_base( tlb, /col, /frame )
label = widget_label( bot_base, value = 'Individual Tests' )
base = widget_base( bot_base, /row, /base_align_center )

list = widget_droplist( base, uvalue = 'Test List', $
  value = [ $
  'Night', $
  'Day', $
  'Sunglint', $
  'Snow or Ice', $
  'Water', $
  'Coast', $
  'Desert', $
  'Land', $
  'Smoke or Dust', $
  'Thin Cirrus VIS', $
  'Shadow', $
  'Thin Cirrus IR', $
  'IR Threshold', $
  'CO2', $
  '6.7 micron', $
  '1.88 micron', $
  '3.7-12 micron', $
  'IR Difference', $
  '3.7-11 micron', $
  'VIS Threshold', $
  'VIS Ratio', $
  '.935/.87 micron', $
  '3.7-3.9 micron', $
  'Temporal Var.', $
  'Spatial Var.' ] )

b = widget_base( bot_base, /row, /align_center )
butt = widget_button( b, value = 'Toggle', uvalue = 'Test Toggle', xsize = 65 )

;- realize widgets

widget_control, tlb, /realize

;- fill colors in draw widgets

for i = 0, n_elements( info.mask_names ) - 1 do begin
  widget_control, draw_ids( i ), get_value = wid
  wset, wid & erase, info.mask_color( i )
endfor

;- set button values

for i = 0, n_elements( info.mask_buttons ) - 1 do begin
  widget_control, info.mask_buttons( i ), set_button = info.mask_select( i )
endfor

;- set droplist selection
;- (note: offset of 6 is applied because first 6 tests defined in
;-  mask_image.pro are not displayed)

widget_control, list, set_droplist_select = info.mask_list - 6

;- desensitize widgets if no cloud mask file is loaded

if info.mask_path eq " " then begin
  widget_control, top_base, sensitive = 0
  widget_control, bot_base, sensitive = 0
endif

;- update image

if info.mask_path ne " " then begin
  info.mask_toggle = 1
  widget_control, top, hourglass = 1, sensitive = 0
  mas_display, info, title = info.title
  widget_control, top, hourglass = 0, sensitive = 1
endif

;- update info structure

info.mask_base = tlb
info.mask_topbase = top_base
info.mask_botbase = bot_base

;- restore info structure

pointer_value, pointer, info, /set, /no_copy

;- store top level base id of caller

widget_control, infobase, set_uvalue = top

;- start event manager

xmanager, "sharp_mask", tlb, cleanup = 'sharp_mask_cleanup'

end
