PRO SHARP_PLOT_EVENT, EVENT

@sharp.inc

;- get info structure

widget_control, event.top, get_uvalue = top
widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- get user value from widget causing event

widget_control, event.id, get_uvalue = uvalue

;- check possible events

case uvalue of

  'TLB' : if event.enter eq 1 then begin
    main_base = info.main_base
    pointer_value, pointer, info, /set, /no_copy
    widget_control, main_base, event_pro = 'sharp_event'
    return
  end

  'PLOT_TLB' : if event.enter eq 1 then begin
    plot_base = info.plot_base
    pointer_value, pointer, info, /set, /no_copy
    widget_control, plot_base, event_pro = 'sharp_plot_event'
    return
  end

  'Region' : begin

    widget_control, event.top, sensitive=0
    
    ;- get user-defined region of interest

    result = my_cw_defroi( info.draw, group = event.top, $
	image_size = [ 716, 716 ], offset = [ 0, 0 ], /restore, $
	xvertices = xvertices, yvertices = yvertices, info = info )

    ;- check if valid region was defined

    if result( 0 ) ne -1 then begin

      widget_control, event.top, hourglass = 1
      
      ;- get data to store in region object

      if info.plot_xdata( 0 ) eq -999.9 then begin
        band = 2
        mas_read, info.path, band, image, date, time1, time2, $
          scene = info.scene, units = info.units_flag
        minv = min( image, max = maxv )
        info.x_bands = [ band, 2, 3, 4 ]
        info.x_operators = [ 0, 0, 0 ]
        info.x_title = string( band, format = '( "(B", i2.2, ")" )' )
        info.plot_xdata = temporary( image )
        info.plot_xmin = minv
        info.plot_xmax = maxv
      endif

      if info.plot_ydata( 0 ) eq -999.9 then begin
        band = 45
        if info.nbands eq 12 then band = 11
        mas_read, info.path, band, image, date, time1, time2, $
          scene = info.scene, units = info.units_flag
        minv = min( image, max = maxv )
        info.y_bands = [ band, 2, 3, 4 ]
        info.y_operators = [ 0, 0, 0 ]
        info.y_title = string( band, format = '( "(B", i2.2, ")" )' )
        info.plot_ydata = temporary( image )
        info.plot_ymin = minv
        info.plot_ymax = maxv
      endif

      ;- extract data within region of interest
      
      x_data = info.plot_xdata( result )
      x_minv = min( x_data, max = x_maxv )
      y_data = info.plot_ydata( result )
      y_minv = min( y_data, max = y_maxv )
      
      ;- convert region result (absolute array index) to x/y array index
      
      sz = size( info.plot_xdata )
      nx = long( sz( 1 ) )
      x_index = result mod nx
      y_index = result / nx

      ;- compute pseudo-centroid
      
      x_centroid = total( x_index ) / float( n_elements( x_index ) )
      y_centroid = total( y_index ) / float( n_elements( y_index ) )
      
      ;- create and store region object
      
      region_data = { $
        result     : result, $
        xvertices  : xvertices, $
        yvertices  : yvertices, $
        x_data     : x_data, $
        x_minv     : x_minv, $
        x_maxv     : x_maxv, $
        y_data     : y_data, $
        y_minv     : y_minv, $
        y_maxv     : y_maxv, $
        x_centroid : x_centroid, $
        y_centroid : y_centroid, $
        x_index    : x_index, $
        y_index    : y_index, $
        pointer     : pointer }
      pointer_value, info.region_ptr( info.region ), region_data, /set, /no_copy

      ;- update the region counter and sensitize the clear and save buttons
      
      info.region = ( info.region + 1 ) < 9
      widget_control, info.clear_id, sensitive = 1
      if not check_demo_mode() then widget_control, info.save_id, sensitive = 1
      
      ;- update the draw and plot windows

      sharp_plot_update_draw, info
      sharp_plot_update_plot, info
      info.plot_xmin = !x.crange( 0 )
      info.plot_xmax = !x.crange( 1 )
      info.plot_ymin = !y.crange( 0 )
      info.plot_ymax = !y.crange( 1 )
	if widget_info( info.x_min_id, /valid_id ) then $
        widget_control, info.x_min_id, set_value = info.plot_xmin
	if widget_info( info.x_max_id, /valid_id ) then $
        widget_control, info.x_max_id, set_value = info.plot_xmax
	if widget_info( info.y_min_id, /valid_id ) then $
        widget_control, info.y_min_id, set_value = info.plot_ymin
	if widget_info( info.y_max_id, /valid_id ) then $
        widget_control, info.y_max_id, set_value = info.plot_ymax

      widget_control, event.top, hourglass = 0

    endif	

    widget_control, event.top, sensitive=1
    
  end

  'Axes' : begin
    widget_control, info.axes_id, sensitive = 0
    pointer_value, pointer, info, /set, /no_copy
    sharp_plot_axes, top, group = event.top
    return
  end
  
  'Clear' : begin
    info.region = ( info.region - 1 ) > 0
    pointer_value, info.region_ptr( info.region ), { result:-1 }, /set, /no_copy
    sharp_plot_update_draw, info
    sharp_plot_update_plot, info
    info.plot_xmin = !x.crange( 0 )
    info.plot_xmax = !x.crange( 1 )
    info.plot_ymin = !y.crange( 0 )
    info.plot_ymax = !y.crange( 1 )
    if widget_info( info.x_min_id, /valid_id ) then $
      widget_control, info.x_min_id, set_value = info.plot_xmin
    if widget_info( info.x_max_id, /valid_id ) then $
      widget_control, info.x_max_id, set_value = info.plot_xmax
    if widget_info( info.y_min_id, /valid_id ) then $
      widget_control, info.y_min_id, set_value = info.plot_ymin
    if widget_info( info.y_max_id, /valid_id ) then $
      widget_control, info.y_max_id, set_value = info.plot_ymax
    if info.region eq 0 then begin
      widget_control, info.clear_id, sensitive = 0
      if not check_demo_mode() then widget_control, info.save_id, sensitive = 0
    endif
  end    

  'Save' : begin

    ;- desensitize all other widgets
    
    widget_control, event.top, sensitive = 0
    widget_control, info.main_base, sensitive = 0

    ;- create dialog for save option buttons

    widget_control, event.id, tlb_get_offset = offset
    tlb = widget_base( group = event.top, $
      xoffset = offset( 0 ) + 4 * 55, yoffset = offset( 1 ) + 25, /column, $
      title = 'Save Regions' )
    base = widget_base( tlb, /row, /exclusive, /align_center, /frame )
    radio1 = widget_button( base, value = 'X/Y Axes', uvalue = 'X/Y Axes', /no_release )
    radio2 = widget_button( base, value = 'All Bands', uvalue = 'All Bands', /no_release )
    base = widget_base( tlb, /row, /align_center )      
    butt = widget_button( base, value = 'OK', uvalue = 'OK', xsize = 55 )
    butt = widget_button( base, value = 'Cancel', uvalue = 'Cancel', xsize = 55 )
    widget_control, tlb, /realize
    if info.save_flag then begin
      widget_control, radio2, /set_button
    endif else begin
      widget_control, radio1, /set_button
    endelse
    save_flag = info.save_flag

    ;- Get an event (this is a hack that is not recommended!)
    
    get_save_event:
    new_event = widget_event(tlb, bad_id=bad_id)
    if not widget_info(tlb, /valid_id) then goto, finish_save
    widget_control, new_event.id, get_uvalue = uvalue

    ;- pointer radio button events
    
    if uvalue eq 'X/Y Axes' then begin
      save_flag = 0
      goto, get_save_event
    endif

    if uvalue eq 'All Bands' then begin
      save_flag = 1
      goto, get_save_event
    endif

    ;- Cancel or OK button was hit, so destroy widgets
    
    widget_control, tlb, /destroy
    
    ;- pointer Cancel button event
    
    if uvalue eq 'Cancel' then goto, finish_save
      
    ;- get output file name

    plot_file = dialog_outputfile( file = info.plot_file, path = info.plot_path, $
      filter = '*.dat', group = event.top, /write, get_path = plot_path, $
      dialog_parent = info.plot_drawid, title = 'Enter plot save filename' )
    if plot_file eq '' then goto, finish_save

    ;- save selected JPG file and path, and save flag
    
    info.plot_file = plot_file
    info.plot_path = plot_path
    info.save_flag = save_flag
              
    ;- turn on hourglass
    
    widget_control, event.top, hourglass = 1

    ;- open output file and print header
    
    openw, outlun, plot_file, /get_lun
    printf, outlun, info.path, format = '( "#Name:   ", a )'
    printf, outlun, info.date, format = '( "#Date:   ", a )'
    printf, outlun, long( info.time1 ), format = '( "#Time1:  ", i6.6 )'
    printf, outlun, long( info.time2 ), format = '( "#Time2:  ", i6.6 )'
    if info.save_flag eq 0 then printf, outlun, info.x_title, format = '( "#X Data: ", a )'
    if info.save_flag eq 0 then printf, outlun, info.y_title, format = '( "#Y Data: ", a )'
    if info.units_flag then begin
	printf, outlun, "#Units:   Radiance (W/m!U2!N/sr/micron) (B01-B50)"
    endif else begin
	printf, outlun, "#Units:   Reflectance (B01-B25), Temperature (K) (B26-B50)'
    endelse
    
    ;- loop through region pointers, and save geolocation information at centroid
    
    printf, outlun, "#Region        Lat        Lon   SolarZen   SolarAzm  SensorZen  SensorAzm"
    for i = 0, n_elements( info.region_ptr ) - 1 do begin
      pointer_value, info.region_ptr( i ), region_data, /no_copy
      if region_data.result( 0 ) ne -1 then begin
        lat    = info.map_lat( region_data.x_centroid, region_data.y_centroid )
        lon    = info.map_lon( region_data.x_centroid, region_data.y_centroid )
        solzen = info.solzen( region_data.x_centroid, region_data.y_centroid )
        solazm = info.solazm( region_data.x_centroid, region_data.y_centroid )
        senzen = info.senzen( region_data.x_centroid, region_data.y_centroid )
        senazm = info.senazm( region_data.x_centroid, region_data.y_centroid )
        printf, outlun, i + 1, lat, lon, solzen, solazm, senzen, senazm, format = '("#",f6.1,6f11.3)'
      endif
      pointer_value, info.region_ptr( i ), region_data, /set, /no_copy
    endfor

    ;- loop through region pointers, and save valid regions
    
    for i = 0, n_elements( info.region_ptr ) - 1 do begin

      ;- get the region data
      
      pointer_value, info.region_ptr( i ), region_data, /no_copy

      if region_data.result( 0 ) ne -1 then begin

        if info.save_flag eq 0 then begin
        
          ;- X/Y axes selected for output
        
          ;- create output array containing [region number, x value, y value]

          outarray = fltarr( 5, n_elements( region_data.x_data ), /nozero )
          outarray( 0, * ) = i + 1
          outarray( 1, * ) = ( region_data.x_data > ( -9999.999 ) ) < ( 9999.999 )
          outarray( 2, * ) = ( region_data.y_data > ( -9999.999 ) ) < ( 9999.999 )
          outarray( 3, * ) = info.map_lat( region_data.result )
          outarray( 4, * ) = info.map_lon( region_data.result )
          
          ;- write to output file
          
          printf, outlun, "#Region     Xdata     Ydata    Lat       Lon"
          printf, outlun, temporary( outarray ), $
            format = '( 1x, f6.1, 1x, f9.3, 1x, f9.3, 1x, f9.4, 1x, f9.4 )'

        endif else begin

          ;- All Bands selected for output        

          ;- get region edges

          xmin = min( region_data.x_index, max = xmax )
          ymin = min( region_data.y_index, max = ymax )

          ;- read region data (all 50 bands)

          scene = [ info.scene(0) + ymin, info.scene(0) + ymax ]
          pixel = xmin
          npixels = xmax - xmin + 1
          region_read, info.path, scene, pixel, npixels, image, units = info.units_flag        
          sz = size( image )
          nbands = sz( 2 )

          ;- create output array
          
          nsamples = n_elements( region_data.x_index )
          ncolumns = nbands + 3
          outarray = fltarr( ncolumns, nsamples )
          outarray( 0, * ) = i + 1
          outarray( ncolumns - 2, * ) = info.map_lat( region_data.result )
          outarray( ncolumns - 1, * ) = info.map_lon( region_data.result )
          
          ;- extract data for each band

          for band = 1, nbands do begin
            subimage = ( reform( image( *, band - 1, * ) ) > ( -9999.999 ) ) < ( 9999.999 )
            outarray( band, * ) = $
              subimage( region_data.x_index - xmin, region_data.y_index - ymin )
          endfor
          
          ;- write to output file

          if nbands eq 12 then begin
          
            outarray = [ outarray( 0, * ), outarray( 2 : *, * ) ]
            labels = replicate( 'B', 11 ) + string( indgen( 11 ) + 2, format = '(i2.2)' )          
            printf, outlun, "#Region ", labels, $
              format = '( a8, 11( 3x, a3, 2x ), "    Lat       Lon"  )'
            printf, outlun, temporary( outarray ), $
              format = '( 12f8.3, 1x, f9.4, 1x, f9.4 )'

          endif else begin

            labels = replicate( 'B', 50 ) + string( indgen( 50 ) + 1, format = '(i2.2)' )          
            printf, outlun, "#Region ", labels, $
              format = '( a8, 50( 3x, a3, 2x ), "    Lat       Lon" )'
            printf, outlun, temporary( outarray ), $
              format = '( 51f8.3, 1x, f9.4, 1x, f9.4 )'

          endelse
          
        endelse
                        
      endif

      ;- restore the region data

      pointer_value, info.region_ptr( i ), region_data, /set, /no_copy

    endfor

    ;- close output file
    
    free_lun, outlun
    
    ;- turn off hourglass and sensitize other widgets

    finish_save:        
    widget_control, event.top, hourglass = 0, sensitive = 1
    widget_control, info.main_base, sensitive = 1

  end
    
  else : print, uvalue
  
end

;- update info structure

finish:
pointer_value, pointer, info, /set, /no_copy

end

;##############################################################################

PRO SHARP_PLOT_DISMISS, EVENT

;- Kill the widget (which causes the cleanup routine to be called)

widget_control, event.top, map = 0
widget_control, event.top, /destroy

END

;##############################################################################

PRO SHARP_PLOT_CLEANUP, ID

;- Get id of top level widget, and return if invalid
;- (this happens when the top level widget is destroyed)

widget_control, id, get_uvalue = top
if not widget_info( top, /valid_id ) then return

;- Get info structure

widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- Set plot off

if ( not info.mask_flag ) and ( not info.his_flag ) then $
  widget_control, info.map_button, sensitive = 1
widget_control, info.plot_button, sensitive = 1
info.plot_flag = 0
sharp_plot_update_draw, info

;- Update info structure

pointer_value, pointer, info, /set, /no_copy

END

;##############################################################################

PRO SHARP_PLOT, TOP, GROUP = GROUP, XOFFSET = XOFFSET, YOFFSET = YOFFSET

rcs_id = "$Id: sharp_plot.pro,v 1.19 1999/06/10 19:17:08 gumley Exp $"

@sharp.inc

;- check keywords

if n_elements( group ) eq 0 then group = top
device, get_screen_size = sz
if n_elements( xoffset ) eq 0 then xoffset = sz( 0 ) / 5
if n_elements( yoffset ) eq 0 then yoffset = sz( 1 ) / 3

;- create base widget

tlb = widget_base( group = group, column = 1, $
  uvalue = 'PLOT_TLB', title = 'Scatterplot' )
infobase = tlb

;- create buttons, draw widget, and text display

base = widget_base( tlb, /row, /frame )
butt = widget_button( base, value = 'Region', uvalue = 'Region', xsize = 55 )
axes_id = widget_button( base, value = 'Axes', uvalue = 'Axes', xsize = 55 )
clear_id = widget_button( base, value = 'Clear', uvalue = 'Clear', xsize = 55 )
save_id = widget_button( base, value = 'Save', uvalue = 'Save', xsize = 55 )
butt = widget_button( base, value = 'Dismiss', uvalue = 'Dismiss', xsize = 55, $
  event_pro = 'sharp_plot_dismiss' )
plot = widget_draw( tlb, scr_xsize = plot_xsize, scr_ysize = plot_ysize, uvalue = 'Plot' )

;- realize widgets

widget_control, tlb, /realize
widget_control, plot, get_value = plot_window
wset, plot_window
erase, 0

;- activate save button only if not in demo mode

widget_control, save_id, sensitive = 0
if not check_demo_mode() then widget_control, save_id, sensitive = 1

;- update info structure

widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy
info.plot_base = tlb
info.plot_drawid = plot
info.plot_window = plot_window
info.clear_id = clear_id
info.axes_id = axes_id
info.save_id = save_id

;- desensitize the clear and save buttons if no plots are defined

if info.region eq 0 then begin
  widget_control, info.clear_id, sensitive = 0
  widget_control, info.save_id, sensitive = 0
endif

;- update the draw and plot windows

sharp_plot_update_draw, info
sharp_plot_update_plot, info, $
  xrange = [ info.plot_xmin, info.plot_xmax ], $
  yrange = [ info.plot_ymin, info.plot_ymax ]

;- restore info structure

pointer_value, pointer, info, /set, /no_copy

;- store top level base id of caller

widget_control, infobase, set_uvalue = top

;- start event manager

xmanager, "sharp_plot", tlb, cleanup = 'sharp_plot_cleanup'

END
