PRO SHARP_PLOT_AXES_EVENT, EVENT

@sharp.inc

;- get info structure

widget_control, event.top, get_uvalue = top
widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- get user value from widget causing event

widget_control, event.id, get_uvalue = uvalue

;- check possible events

case uvalue of

  'Update' : begin
    widget_control, info.x_min_id, get_value = xmin
    widget_control, info.x_max_id, get_value = xmax
    widget_control, info.y_min_id, get_value = ymin
    widget_control, info.y_max_id, get_value = ymax
    widget_control, info.x_min_id, set_value = xmin
    widget_control, info.x_max_id, set_value = xmax
    widget_control, info.y_min_id, set_value = ymin
    widget_control, info.y_max_id, set_value = ymax
    info.plot_xmin = xmin
    info.plot_xmax = xmax
    info.plot_ymin = ymin
    info.plot_ymax = ymax
    sharp_plot_update_plot, info, $
      xrange = [ info.plot_xmin, info.plot_xmax ], $
      yrange = [ info.plot_ymin, info.plot_ymax ]
  end

  'X_Formula' : begin

    ;- get new x axis formula
    
    bands = info.x_bands
    operators = info.x_operators
    widget_control, event.id, tlb_get_offset = offset
    formula, info.path, result, group = event.top, $
      xoffset = offset( 0 ) + 75, yoffset = offset( 1 ) + 25, $
      bands = bands, operators = operators, title = title, $
      scene = info.scene, units = info.units_flag

    if result(0) ne -1.0 then begin

      ;- update label
      
      widget_control, info.x_label_id, set_value = "X Axis: " + title
      
      ;- update x axis data for whole of image
      
      info.plot_xdata = temporary( result )
      info.x_bands = bands
      info.x_operators = operators
      info.x_title = title

      ;- update x axis data for each region
      
	for i = 0, 9 do begin

	  pointer_value, info.region_ptr( i ), region_data, /no_copy

	  if region_data.result( 0 ) ne -1 then begin

          ;- extract data within region of interest

          x_data = info.plot_xdata( region_data.result )
          x_minv = min( x_data, max = x_maxv )

          ;- update region object

          region_data = { $
      	result     : region_data.result, $
      	xvertices  : region_data.xvertices, $
      	yvertices  : region_data.yvertices, $
      	x_data     : x_data, $
      	x_minv     : x_minv, $
      	x_maxv     : x_maxv, $
      	y_data     : region_data.y_data, $
      	y_minv     : region_data.y_minv, $
      	y_maxv     : region_data.y_maxv, $
            x_centroid : region_data.x_centroid, $
            y_centroid : region_data.y_centroid, $
            x_index    : region_data.x_index, $
            y_index    : region_data.y_index, $
      	pointer     : region_data.pointer }

	  endif

	  pointer_value, info.region_ptr( i ), region_data, /set, /no_copy

	endfor

      sharp_plot_update_plot, info, $
        yrange = [ info.plot_ymin, info.plot_ymax ]
      info.plot_xmin = !x.crange( 0 )
      info.plot_xmax = !x.crange( 1 )
      widget_control, info.x_min_id, set_value = info.plot_xmin
      widget_control, info.x_max_id, set_value = info.plot_xmax

    endif

  end

  'Y_Formula' : begin

    ;- get new y axis formula
    
    bands = info.y_bands
    operators = info.y_operators
    widget_control, event.id, tlb_get_offset = offset
    formula, info.path, result, group = event.top, $
      xoffset = offset( 0 ) + 75, yoffset = offset( 1 ) + 25, $
      bands = bands, operators = operators, title = title, $
      scene = info.scene, units = info.units_flag

    if result(0) ne -1.0 then begin

      ;- update label
      
      widget_control, info.y_label_id, set_value = "Y Axis: " + title

      ;- update y axis data for whole of image

      info.plot_ydata = temporary( result )
      info.y_bands = bands
      info.y_operators = operators
      info.y_title = title
      
      ;- update y axis data for each region
      
	for i = 0, 9 do begin

	  pointer_value, info.region_ptr( i ), region_data, /no_copy

	  if region_data.result( 0 ) ne -1 then begin

          ;- extract data within region of interest

          y_data = info.plot_ydata( region_data.result )
          y_minv = min( y_data, max = y_maxv )

          ;- update region object

          region_data = { $
      	result     : region_data.result, $
      	xvertices  : region_data.xvertices, $
      	yvertices  : region_data.yvertices, $
      	x_data     : region_data.x_data, $
      	x_minv     : region_data.x_minv, $
      	x_maxv     : region_data.x_maxv, $
      	y_data     : y_data, $
      	y_minv     : y_minv, $
      	y_maxv     : y_maxv, $
            x_centroid : region_data.x_centroid, $
            y_centroid : region_data.y_centroid, $
            x_index    : region_data.x_index, $
            y_index    : region_data.y_index, $
      	pointer     : region_data.pointer }

	  endif

	  pointer_value, info.region_ptr( i ), region_data, /set, /no_copy

	endfor

      sharp_plot_update_plot, info, $
        xrange = [ info.plot_xmin, info.plot_xmax ]
      info.plot_ymin = !y.crange( 0 )
      info.plot_ymax = !y.crange( 1 )
      widget_control, info.y_min_id, set_value = info.plot_ymin
      widget_control, info.y_max_id, set_value = info.plot_ymax

    endif
      
  end

  else : print, uvalue
  
end

;- restore info structure

pointer_value, pointer, info, /set, /no_copy

END

;##############################################################################

PRO SHARP_PLOT_AXES_DISMISS, EVENT

;- Kill the widget (which causes the cleanup routine to be called)

widget_control, event.top, map = 0
widget_control, event.top, /destroy

END

;##############################################################################

PRO SHARP_PLOT_AXES_CLEANUP, ID

;- Get id of top level widget, and return if invalid
;- (this happens when the top level widget is destroyed)

widget_control, id, get_uvalue = top
if not widget_info( top, /valid_id ) then return

;- Get info structure

widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy

;- Set plot axes off

if widget_info( info.axes_id, /valid_id ) then $
  widget_control, info.axes_id, sensitive = 1
info.x_min_id = -1L
info.x_max_id = -1L
info.y_min_id = -1L
info.y_max_id = -1L

;- Update info structure

pointer_value, pointer, info, /set, /no_copy

END

;##############################################################################

PRO SHARP_PLOT_AXES, TOP, GROUP = GROUP, XOFFSET = XOFFSET, YOFFSET = YOFFSET

rcs_id = "$Id: sharp_plot_axes.pro,v 1.6 1999/06/10 18:57:47 gumley Exp $"
  
;- check keywords

if n_elements( group ) eq 0 then group = top
device, get_screen_size = sz
if n_elements( xoffset ) eq 0 then xoffset = sz( 0 ) / 4
if n_elements( yoffset ) eq 0 then yoffset = sz( 1 ) / 4

;- create top level base

tlb = widget_base( column = 1, group = group, title = 'Scatterplot Axes', $
  xoffset = xoffset, yoffset = yoffset )
infobase = tlb

;- create widgets

top_base = widget_base( tlb, column = 1, /align_left, /frame ) 

x_label_id = widget_label( top_base, value = '       X Axis: (B02)       ' )
base1 = widget_base( top_base, /row )
formula = widget_button( base1, value = 'Formula', uvalue = 'X_Formula', $
  xsize = 60 )
x_min_id = cw_field( base1, value = minv, xsize = 6, title = 'Minimum', /float )
x_max_id = cw_field( base1, value = maxv, xsize = 6, title = 'Maximum', /float )

y_label_id = widget_label( top_base, value = '       Y Axis: (B45)       ' )
base2 = widget_base( top_base, /row )
formula = widget_button( base2, value = 'Formula', uvalue = 'Y_Formula', $
  xsize = 60 )
y_min_id = cw_field( base2, value = minv, xsize = 6, title = 'Minimum', /float )
y_max_id = cw_field( base2, value = maxv, xsize = 6, title = 'Maximum', /float )

bot_base = widget_base( tlb, /row, /align_center )
butt = widget_button( bot_base, value = 'Update', uvalue = 'Update', xsize = 60 )
butt = widget_button( bot_base, value = 'Dismiss', uvalue = 'Dismiss', xsize = 60, $
  event_pro = 'sharp_plot_axes_dismiss' )

;- realize widgets

widget_control, tlb, /realize

;- update info structure

widget_control, top, get_uvalue = pointer
pointer_value, pointer, info, /no_copy
info.x_min_id = x_min_id
info.x_max_id = x_max_id
info.x_label_id = x_label_id
info.y_min_id = y_min_id
info.y_max_id = y_max_id
info.y_label_id = y_label_id

;- update labels

if info.x_title ne " " then $
  widget_control, info.x_label_id, set_value = "X Axis: " + info.x_title
widget_control, info.x_min_id, set_value = info.plot_xmin
widget_control, info.x_max_id, set_value = info.plot_xmax
if info.y_title ne " " then $
  widget_control, info.y_label_id, set_value = "Y Axis: " + info.y_title
widget_control, info.y_min_id, set_value = info.plot_ymin
widget_control, info.y_max_id, set_value = info.plot_ymax

;- restore info structure

pointer_value, pointer, info, /set, /no_copy

;- store top level base id of caller

widget_control, infobase, set_uvalue = top

;- start event manager

xmanager, "sharp_plot_axes", tlb, cleanup = 'sharp_plot_axes_cleanup'

END
