;##############################################################################

PRO SHARP_PLOT_UPDATE_PLOT, INFO, $
  REGION = REGION, XRANGE = XRANGE, YRANGE = YRANGE

rcs_id = "$Id: sharp_plot_update_plot.pro,v 1.3 1999/06/10 16:41:08 gumley Exp $"

@sharp.inc

if not keyword_set( region ) then region = [ 0.0, 0.0, 1.0, 1.0 ]

if !d.name ne 'PS' then select_font, size = 12, /bold

;- extract items from the info structure

plot_window = info.plot_window
region_ptr = info.region_ptr
x_title = info.x_title
y_title = info.y_title

;- define 10 graphics colors

colors = [ 5, 4, 6, 1, 2, 3, 14, 9, 10, 15 ]

;- get axis ranges for plot

x_minv = 1.0e20
x_maxv = -1.0e20
y_minv = 1.0e20
y_maxv = -1.0e20
for i = 0, 9 do begin
  pointer_value, region_ptr( i ), region_data, /no_copy
  if region_data.result( 0 ) ne -1 then begin
    x_minv = x_minv < region_data.x_minv
    x_maxv = x_maxv > region_data.x_maxv
    y_minv = y_minv < region_data.y_minv
    y_maxv = y_maxv > region_data.y_maxv
  endif
  pointer_value, region_ptr( i ), region_data, /set, /no_copy
endfor

if not keyword_set( xrange ) then begin
  xdelta = x_maxv - x_minv
  xrange = [ x_minv - 0.1 * xdelta, x_maxv + 0.1 * xdelta ]
endif

if not keyword_set( yrange ) then begin
  ydelta = y_maxv - y_minv
  yrange = [ y_minv - 0.1 * ydelta, y_maxv + 0.1 * ydelta ]
endif

;- set title

if info.units_flag then begin
  title = 'Radiance (W/m!U2!N/sr/micron) (B01-B50)'
endif else begin
  title = 'Reflectance (B01-B25), Temperature (K) (B26-B50)'
endelse

;- update regions in plot window

if !d.name ne 'PS' then begin
  wset, plot_window
  erase, 0
endif

symbols = [ 1, 2, 4, 5, 6, 7, 1, 2, 4, 5 ]

if !d.name eq 'PS' then tvlct, 0B, 0B, 0B, 7

thick = 1.0
if !d.name eq 'PS' then thick = 1.5

for i = 0, 9 do begin

  pointer_value, region_ptr( i ), region_data, /no_copy

  if region_data.result( 0 ) ne -1 then begin

    ;- establish plot coordinates
    
    x0 = 0.15
    y0 = 1.0 - 0.85 * float( plot_xsize ) / float( plot_ysize )
    x1 = 0.95
    y1 = 0.95
    if i eq 0 then begin
      pos = [ x0, y0, x1, y1 ]
      pos = [ convert_region( pos( 0 ), pos( 1 ), region ), $
              convert_region( pos( 2 ), pos( 3 ), region ) ]
	plot, [ 0 ], pos = pos, /noerase, xrange = xrange, yrange = yrange, $
        /nodata, color = 7, xtitle = x_title, ytitle = y_title, title = title, $
        xcharsize = 1.25, ycharsize = 1.25, $
        xthick = thick, ythick = thick, $
        xstyle = 1, ystyle = 1
    endif
    
    ;- plot the points
    
    plots, region_data.x_data, region_data.y_data, $
      color = colors( i ), psym = symbols( i ), symsize = 1.25, noclip = 0

    ;- compute statistics
    
    stddev, region_data.x_data, xmean, xvariance, xsigma
    stddev, region_data.y_data, ymean, yvariance, ysigma
    x = 0.5
    y = 0.185 - 0.035 * i
    pos = convert_region( x, y, region )
    label = string( xmean, xsigma, ymean, ysigma, $
      format = '( "XAVG:", f7.2, ", XSIG:", f7.2, ", YAVG:", f7.2, ", YSIG:", f7.2 )' )
    xyouts, pos(0), pos(1), label, /normal, charsize = 1.25, color = 7, align = 0.5

    x = 0.025
    y = 0.185 - 0.035 * i
    pos = convert_region( x, y, region )
    xyouts, pos(0), pos(1), strcompress( i + 1, /remove_all ), $
      /normal, color = colors( i )
    
  endif else begin
  
    if i eq 0 then begin
      xyouts, 0.5, 0.6, '!17Hit Region button,!X', $
	  /normal, align = 0.5, charsize = 2.0, color = 4
      xyouts, 0.5, 0.5, '!17Left click and drag on image,!X', $
	  /normal, align = 0.5, charsize = 2.0, color = 4
      xyouts, 0.5, 0.4, '!17Hit OK button!X', $
	  /normal, align = 0.5, charsize = 2.0, color = 4
    endif
    
  endelse    

  pointer_value, region_ptr( i ), region_data, /set, /no_copy

endfor

if !d.name eq 'PS' then tvlct, 255B, 255B, 255B, 7

if !d.name ne 'PS' then select_font, size = 14, /bold

END
