FUNCTION UNIQUE, ARRAY

;+
; PURPOSE:
;    To return the unique elements of an array in the same order as they
;    appear on input (i.e. unsorted).
;
; USAGE:
;    RESULT = UNIQUE(ARRAY)
;
; INPUT:
;    ARRAY     Array of input data (does not need to be sorted).
;
; OUTPUT:
;    RESULT    Array of unique elements, in the same order as they
;              appear in the input array.
;
; EXAMPLE:
; print, unique([3,2,1,1,2,3,4,5,4,3,2,3,4])
; print, unique(['t.dat','t.dat','d.txt','d2.txt','a.exe','t.dat','d.txt'])
;
; REVISED:
; Liam.Gumley@ssec.wisc.edu
; $Id: unique.pro,v 1.1 1999/06/17 18:39:08 gumley Exp $
;-

rcs_id = '$Id: unique.pro,v 1.1 1999/06/17 18:39:08 gumley Exp $

;- Check arguments

if n_params() ne 1 then message, 'Usage: RESULT = UNIQUE(ARRAY)'
if n_elements(array) eq 0 then message, 'Argument ARRAY is undefined'

;- Get sort index, and unique index for sorted data

sindex = sort(array)
uindex = uniq(array[sindex])

;- Create unique index array for unsorted data

newindex = sindex[uindex]
newindex = newindex[sort(newindex)]

;- Return array of unsorted unique data

return, array[newindex]

END
